@@ -38,6 +38,14 @@
 extern "C" {
 #endif /* __cplusplus */
 
+/* The package name we need to pass to dgettext to get GTK+'s translated
+   strings. */
+#define GLADE_GTK_GETTEXT_PACKAGE	"gtk20"
+
+
+  /* The index of the dialog hint in GbTypeHintChoices. */
+#define GLADE_TYPE_HINT_DIALOG_INDEX	1
+
 /* This is the error margin we use for floating-point comparisons. */
 #define GLADE_EPSILON	0.000001
 
@@ -53,6 +61,9 @@ extern const gchar *GladeParentKey;
 extern const gchar *GladeButtonStockIDKey;
 extern const gchar *GladeDialogResponseIDKey;
 
+extern const gchar *GladeToolButtonStockIDKey;
+extern const gchar *GladeToolButtonIconKey;
+
 /* The moniker key, only used for Bonobo controls. */
 extern const gchar *Moniker;
 
@@ -100,7 +111,6 @@ extern const gchar *GladeChildBonoboWindowDock;
 extern const gchar *GladeChildBonoboWindowAppBar;
 
 /* These aren't saved in the XML. */
-extern const gchar *GladeChildToolbarButton;
 extern const gchar *GladeChildCListTitle;
 
 
@@ -155,24 +165,24 @@ void	    gb_button_get_standard_properties	(GtkWidget * widget,
 						 gchar *stock_id_p,
 						 gchar *label_p,
 						 gchar *icon_p,
-						 gchar *relief_p);
+						 gchar *relief_p,
+						 gchar *focus_on_click_p);
 void	    gb_button_set_standard_properties	(GtkWidget * widget,
 						 GbWidgetSetArgData * data,
 						 gchar *stock_id_p,
 						 gchar *label_p,
 						 gchar *icon_p,
-						 gchar *relief_p);
+						 gchar *relief_p,
+						 gchar *focus_on_click_p);
 void	    gb_button_create_popup_menu		(GtkWidget * widget,
 						 GbWidgetCreateMenuData * data);
 void	    gb_button_find_radio_group		(GtkWidget *widget,
 						 GladeFindGroupData *find_data);
 void	    gb_button_write_standard_source	(GtkWidget * widget,
-						 GbWidgetWriteSourceData * data);
+						 GbWidgetWriteSourceData * data,
+						 const gchar *label_p);
 void	    gb_button_destroy			(GtkWidget * widget,
 						 GbWidgetDestroyData * data);
-gboolean    gb_button_toolbar_find_child_widgets(GtkWidget *widget,
-						 GtkWidget **label,
-						 GtkWidget **icon);
 
 char*	    gb_dialog_response_id_to_string	(gint response_id);
 gint	    gb_dialog_response_id_from_string	(const gchar *response_id);
@@ -266,16 +276,25 @@ void	    gb_table_update_placeholders	(GtkWidget * table,
 						 gint rows,
 						 gint cols);
 
-GtkWidget * gb_toolbar_new_toolbar_button	(GbWidgetNewData * data,
-						 GtkToolbarChildType type);
-void	    gb_toolbar_insert_toolbar_child	(GtkToolbar * toolbar,
-						 GtkWidget * new_child,
-						 gint pos);
-gboolean    gb_toolbar_is_toolbar_button	(GtkWidget * widget);
-gboolean    gb_toolbar_get_new_toolbar_group	(GtkWidget * toolbar,
-						 GtkWidget * widget);
-void	    gb_toolbar_create_toolbar_button_popup_menu	(GtkWidget *widget,
-						 GbWidgetCreateMenuData *data);
+void	    gb_tool_button_get_standard_properties (GtkWidget *widget,
+						    GbWidgetGetArgData * data,
+						    gchar *stock_id_p,
+						    gchar *label_p,
+						    gchar *icon_p,
+						    gchar *visible_horz_p,
+						    gchar *visible_vert_p,
+						    gchar *is_important_p);
+void	    gb_tool_button_set_standard_properties (GtkWidget *widget,
+						    GbWidgetSetArgData * data,
+						    gchar *stock_id_p,
+						    gchar *label_p,
+						    gchar *icon_p,
+						    gchar *visible_horz_p,
+						    gchar *visible_vert_p,
+						    gchar *is_important_p);
+void	    gb_tool_button_destroy		(GtkWidget * widget,
+						 GbWidgetDestroyData * data);
+
 
 void	    gb_window_create_standard_properties(GtkWidget * widget,
 						 GbWidgetCreateArgData * data,
@@ -292,7 +311,13 @@ void	    gb_window_create_standard_properties(GtkWidget * widget,
 						 gchar *wmclass_p,
 						 gchar *resizable_p,
 						 gchar *destroy_with_parent_p,
-						 gchar *icon_p);
+						 gchar *icon_p,
+						 gchar *role_p,
+						 gchar *type_hint_p,
+						 gchar *skip_taskbar_p,
+						 gchar *skip_pager_p,
+						 gchar *decorated_p,
+						 gchar *gravity_p);
 void	    gb_window_get_standard_properties	(GtkWidget * widget,
 						 GbWidgetGetArgData * data,
 						 gchar *title_p,
@@ -308,7 +333,13 @@ void	    gb_window_get_standard_properties	(GtkWidget * widget,
 						 gchar *wmclass_p,
 						 gchar *resizable_p,
 						 gchar *destroy_with_parent_p,
-						 gchar *icon_p);
+						 gchar *icon_p,
+						 gchar *role_p,
+						 gchar *type_hint_p,
+						 gchar *skip_taskbar_p,
+						 gchar *skip_pager_p,
+						 gchar *decorated_p,
+						 gchar *gravity_p);
 void	    gb_window_set_standard_properties	(GtkWidget * widget,
 						 GbWidgetSetArgData * data,
 						 gchar *title_p,
@@ -324,7 +355,13 @@ void	    gb_window_set_standard_properties	(GtkWidget * widget,
 						 gchar *wmclass_p,
 						 gchar *resizable_p,
 						 gchar *destroy_with_parent_p,
-						 gchar *icon_p);
+						 gchar *icon_p,
+						 gchar *role_p,
+						 gchar *type_hint_p,
+						 gchar *skip_taskbar_p,
+						 gchar *skip_pager_p,
+						 gchar *decorated_p,
+						 gchar *gravity_p);
 void	    gb_window_write_standard_source	(GtkWidget * widget,
 						 GbWidgetWriteSourceData *data,
 						 gchar *title_p,
@@ -340,7 +377,21 @@ void	    gb_window_write_standard_source	(GtkWidget * widget,
 						 gchar *wmclass_p,
 						 gchar *resizable_p,
 						 gchar *destroy_with_parent_p,
-						 gchar *icon_p);
+						 gchar *icon_p,
+						 gchar *role_p,
+						 gchar *type_hint_p,
+						 gchar *skip_taskbar_p,
+						 gchar *skip_pager_p,
+						 gchar *decorated_p,
+						 gchar *gravity_p);
+void	    gb_window_destroy			(GtkWidget * widget,
+						 GbWidgetDestroyData * data);
+
+
+gboolean    gb_set_scroll_adjustments_hook	(GSignalInvocationHint  *ihint,
+						 guint			n_param_values,
+						 const GValue	       *param_values,
+						 gpointer		data);
 
 
 #ifdef __cplusplus
