@@ -16,6 +16,7 @@
 #define GB_PARAM_INDENT		40
 #define GB_PARAM_TYPE_WIDTH	16
 
+static GHashTable *glade_signal_hash = NULL;
 
 static void gb_widget_write_signals_source (GtkWidget * widget,
 					    GbWidgetWriteSourceData * data);
@@ -27,7 +28,8 @@ static void gb_widget_write_signal_connection_source (GbWidgetWriteSourceData *
 						      const gchar *handler);
 static gchar *get_type_name (GtkType type, gboolean * is_pointer);
 static gchar *get_gdk_event (gchar * signal_name);
-static gchar **lookup_signal_arg_names (gchar * type, gchar * signal_name);
+static gchar **lookup_signal_arg_names (gchar * type, gchar * signal_name,
+					gint num_args_expected);
 static void gb_widget_write_accelerators_source (GtkWidget * widget,
 					    GbWidgetWriteSourceData * data);
 
@@ -101,10 +103,11 @@ gb_widget_write_source (GtkWidget * widget,
 	      /* We don't create standard widgets for Bonobo controls, for
 		 now. There may be a better way to get default values for
 		 them. */
-	      else if (!BONOBO_IS_WIDGET (widget))
+	      else if (!data->standard_widget && !BONOBO_IS_WIDGET (widget))
 		data->standard_widget = GTK_WIDGET (g_object_newv (gtk_type_from_name (class_id), 0, NULL));
 #else
-	      data->standard_widget = GTK_WIDGET (g_object_newv (gtk_type_from_name (class_id), 0, NULL));
+	      if (!data->standard_widget)
+		data->standard_widget = GTK_WIDGET (g_object_newv (gtk_type_from_name (class_id), 0, NULL));
 #endif
 
 	      if (data->standard_widget)
@@ -287,19 +290,19 @@ gb_widget_write_standard_source (GtkWidget * widget,
 
   if (wdata->flags & (GLADE_WIDTH_SET | GLADE_HEIGHT_SET))
     {
-      width = wdata->flags & GLADE_WIDTH_SET ? wdata->width : -2;
-      height = wdata->flags & GLADE_HEIGHT_SET ? wdata->height : -2;
+      width = wdata->flags & GLADE_WIDTH_SET ? wdata->width : -1;
+      height = wdata->flags & GLADE_HEIGHT_SET ? wdata->height : -1;
 
       /* GTK BUG WORKAROUND - a combo should manage the size of its entry.
 	 I think this isn't needed any more (GTK+ 1.2.3). */
 #if 0
       if (GTK_IS_COMBO (widget))
 	source_add(data,
-		   "  gtk_widget_set_usize (GTK_COMBO (%s)->entry, %i, %i);\n",
-		   data->wname, width - 16 < 0 ? -2 : width - 16, height);
+		   "  gtk_widget_set_size_request (GTK_COMBO (%s)->entry, %i, %i);\n",
+		   data->wname, width - 16 < 0 ? -1 : width - 16, height);
 #endif
 
-      source_add (data, "  gtk_widget_set_usize (%s, %i, %i);\n",
+      source_add (data, "  gtk_widget_set_size_request (%s, %i, %i);\n",
 		  data->wname, width, height);
     }
 
@@ -328,8 +331,7 @@ gb_widget_write_standard_source (GtkWidget * widget,
       if (can_focus)
 	source_add (data, "  GTK_WIDGET_SET_FLAGS (%s, GTK_CAN_FOCUS);\n",
 		    data->wname);
-      /* SPECIAL CODE: toolbar widgets can't get focus by default. */
-      else if (!gb_toolbar_is_toolbar_button (widget))
+      else
 	source_add (data, "  GTK_WIDGET_UNSET_FLAGS (%s, GTK_CAN_FOCUS);\n",
 		    data->wname);
     }
@@ -363,13 +365,33 @@ gb_widget_write_standard_source (GtkWidget * widget,
 	data->default_widget = g_strdup (data->wname);
     }
 
-  /* SPECIAL CODE: toolbar children use the toolbar's tooltips. */
-  if (wdata->tooltip && (!widget->parent || !GTK_IS_TOOLBAR (widget->parent)))
+  if (wdata->tooltip)
     {
+      gboolean translatable, context;
+      gchar *comments;
+
+      glade_util_get_translation_properties (widget, GbTooltip, &translatable,
+					     &comments, &context);
+      source_add_translator_comments (data, translatable, comments);
+
       data->need_tooltips = TRUE;
-      source_add (data, "  gtk_tooltips_set_tip (tooltips, %s, %s, NULL);\n",
-		  data->wname,
-		  source_make_string (wdata->tooltip, data->use_gettext));
+
+      /* GtkToolItem subclasses use a special function, since otherwise they
+	 wouldn't work as they don't have a window. */
+      if (GTK_IS_TOOL_ITEM (widget))
+	{
+	  source_add (data,
+		      "  gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (%s), tooltips, %s, NULL);\n",
+		      data->wname,
+		      source_make_string_full (wdata->tooltip, data->use_gettext && translatable, context));
+	}
+      else
+	{
+	  source_add (data,
+		      "  gtk_tooltips_set_tip (tooltips, %s, %s, NULL);\n",
+		      data->wname,
+		      source_make_string_full (wdata->tooltip, data->use_gettext && translatable, context));
+	}
     }
 
   if (!GTK_WIDGET_NO_WINDOW (widget))
@@ -564,7 +586,14 @@ gb_widget_write_signal_handler_source (GtkWidget *widget,
 
 
   signal_id = gtk_signal_lookup (signal_name, GTK_OBJECT_TYPE (widget));
-  g_return_if_fail (signal_id != 0);
+  /* If a project is converted from GTK+ 1.2.x to 2.x.x it may have some
+     signals which aren't valid any more. So we output a warning. */
+  if (signal_id == 0)
+    {
+      g_warning ("Invalid signal: '%s' for widget: '%s'", signal_name,
+		 data->wname);
+      return;
+    }
 
   g_signal_query (signal_id, &query_info);
 
@@ -588,31 +617,6 @@ gb_widget_write_signal_handler_source (GtkWidget *widget,
 	*pos++ = ' ';
     }
 
-  widget_num = 0;
-
-  /* Output the signal object type and the argument name. We assume the
-     type is a pointer - I think that is OK. We remove "Gtk" and convert
-     to lower case for the argument name. */
-  object_name = (char*) g_type_name (query_info.itype);
-  sprintf (pos, "(%s ", object_name);
-  pos += strlen (pos);
-  object_name_len = strlen (object_name);
-  if (object_name_len + 1 < GB_PARAM_TYPE_WIDTH)
-    {
-      num_spaces = GB_PARAM_TYPE_WIDTH - object_name_len - 1;
-      for (i = 0; i < num_spaces; i++)
-	*pos++ = ' ';
-    }
-  object_arg = (!strncmp (object_name, "Gtk", 3)) ? object_name + 3 : object_name;
-  object_arg_start = pos;
-  sprintf (pos, "*%s", object_arg);
-  pos += strlen (pos);
-  g_strdown (object_arg_start);
-  if (!strcmp (object_arg_start, "widget"))
-    widget_num++;
-  sprintf (pos, ",\n");
-  pos += strlen (pos);
-
   /* Convert signal name to use underscores rather than dashes '-'. */
   signal_name_copy = g_strdup (query_info.signal_name);
   for (i = 0; signal_name_copy[i]; i++)
@@ -622,7 +626,44 @@ gb_widget_write_signal_handler_source (GtkWidget *widget,
     }
 
   /* Output the signal parameters. */
-  arg_names = lookup_signal_arg_names (object_name, signal_name_copy);
+  object_name = (char*) g_type_name (query_info.itype);
+  arg_names = lookup_signal_arg_names (object_name, signal_name_copy,
+				       query_info.n_params + 1);
+
+
+  widget_num = 0;
+
+  /* Output the signal object type and the argument name. We assume the
+     type is a pointer - I think that is OK. We remove "Gtk" and convert
+     to lower case for the argument name. */
+  sprintf (pos, "(%s ", object_name);
+  pos += strlen (pos);
+  object_name_len = strlen (object_name);
+  if (object_name_len + 1 < GB_PARAM_TYPE_WIDTH)
+    {
+      num_spaces = GB_PARAM_TYPE_WIDTH - object_name_len - 1;
+      for (i = 0; i < num_spaces; i++)
+	*pos++ = ' ';
+    }
+
+  if (arg_names)
+    {
+      sprintf (pos, "*%s,\n", arg_names[0]);
+      pos += strlen (pos);
+    }
+  else
+    {
+      object_arg = (!strncmp (object_name, "Gtk", 3)) ? object_name + 3 : object_name;
+      object_arg_start = pos + 1;
+      sprintf (pos, "*%s", object_arg);
+      pos += strlen (pos);
+      g_strdown (object_arg_start);
+      if (!strncmp (object_arg_start, "widget", 6))
+	widget_num = 2;
+      sprintf (pos, ",\n");
+      pos += strlen (pos);
+    }
+
 
   param_num = 1;
   event_num = callback_num = 0;
@@ -631,9 +672,11 @@ gb_widget_write_signal_handler_source (GtkWidget *widget,
       for (i = 0; i < GB_PARAM_INDENT; i++)
 	*pos++ = ' ';
 
-      if (arg_names)
+      /* If the arg name includes the type (with a space between!), just
+	 output that. */
+      if (arg_names && strchr (arg_names[param + 1], ' '))
 	{
-	  sprintf (pos, "%s,\n", arg_names[param]);
+	  sprintf (pos, "%s,\n", arg_names[param + 1]);
 	  pos += strlen (pos);
 	}
       else
@@ -674,18 +717,36 @@ gb_widget_write_signal_handler_source (GtkWidget *widget,
 	      for (i = 0; i < num_spaces; i++)
 		*pos++ = ' ';
 	    }
-	  if (!arg_num || *arg_num == 0)
-	    sprintf (pos, "%s%s,\n", is_pointer ? "*" : " ", arg_name);
+
+	  if (arg_names)
+	    {
+	      sprintf (pos, "%s%s,\n", is_pointer ? "*" : " ",
+		       arg_names[param + 1]);
+	      pos += strlen (pos);
+	    }
 	  else
-	    sprintf (pos, "%s%s%i,\n", is_pointer ? "*" : " ", arg_name,
-		     *arg_num);
-	  pos += strlen (pos);
+	    {
+	      if (!arg_num || *arg_num == 0)
+		sprintf (pos, "%s%s,\n", is_pointer ? "*" : " ", arg_name);
+	      else
+		sprintf (pos, "%s%s%i,\n", is_pointer ? "*" : " ", arg_name,
+			 *arg_num);
+	      pos += strlen (pos);
 	  
-	  if (arg_num)
-	    *arg_num += 1;
+	      if (arg_num)
+		{
+		  if (*arg_num == 0)
+		    *arg_num = 2;
+		  else
+		    *arg_num += 1;
+		}
+	    }
 	}
     }
 
+  if (arg_names)
+    g_strfreev (arg_names);
+
   /* Add the final user_data parameter, common to all handlers. */
   for (i = 0; i < GB_PARAM_INDENT; i++)
     *pos++ = ' ';
@@ -721,18 +782,19 @@ gb_widget_write_signal_connection_source (GbWidgetWriteSourceData * data,
       if (connect_after)
 	{
 	  source_add_to_buffer (data, GLADE_SIGNAL_CONNECTIONS,
-				"  gtk_signal_connect_object_after (GTK_OBJECT (%s), \"%s\",\n"
-				"                                   GTK_SIGNAL_FUNC (%s),\n"
-				"                                   GTK_OBJECT (%s));\n",
+				"  g_signal_connect_data ((gpointer) %s, \"%s\",\n"
+				"                         G_CALLBACK (%s),\n"
+				"                         GTK_OBJECT (%s),\n"
+				"                         NULL, G_CONNECT_AFTER | G_CONNECT_SWAPPED);\n",
 				data->wname, signal_name, handler,
 				connect_object);
 	}
       else
 	{
 	  source_add_to_buffer (data, GLADE_SIGNAL_CONNECTIONS,
-				"  gtk_signal_connect_object (GTK_OBJECT (%s), \"%s\",\n"
-				"                             GTK_SIGNAL_FUNC (%s),\n"
-				"                             GTK_OBJECT (%s));\n",
+				"  g_signal_connect_swapped ((gpointer) %s, \"%s\",\n"
+				"                            G_CALLBACK (%s),\n"
+				"                            GTK_OBJECT (%s));\n",
 				data->wname, signal_name, handler,
 				connect_object);
 	}
@@ -742,18 +804,18 @@ gb_widget_write_signal_connection_source (GbWidgetWriteSourceData * data,
       if (connect_after)
 	{
 	  source_add_to_buffer (data, GLADE_SIGNAL_CONNECTIONS,
-				"  gtk_signal_connect_after (GTK_OBJECT (%s), \"%s\",\n"
-				"                            GTK_SIGNAL_FUNC (%s),\n"
-				"                            %s);\n",
+				"  g_signal_connect_after ((gpointer) %s, \"%s\",\n"
+				"                          G_CALLBACK (%s),\n"
+				"                          %s);\n",
 				data->wname, signal_name, handler,
 				handler_data ? handler_data : "NULL");
 	}
       else
 	{
 	  source_add_to_buffer (data, GLADE_SIGNAL_CONNECTIONS,
-				"  gtk_signal_connect (GTK_OBJECT (%s), \"%s\",\n"
-				"                      GTK_SIGNAL_FUNC (%s),\n"
-				"                      %s);\n",
+				"  g_signal_connect ((gpointer) %s, \"%s\",\n"
+				"                    G_CALLBACK (%s),\n"
+				"                    %s);\n",
 				data->wname, signal_name, handler,
 				handler_data ? handler_data : "NULL");
 	}
@@ -803,6 +865,11 @@ get_type_name (GtkType type, gboolean * is_pointer)
        return 'gpointer' just in case. */
     return "gpointer";
 
+  case G_TYPE_PARAM:
+    /* A GParam is really a GParamSpec*. */
+    *is_pointer = TRUE;
+    return "GParamSpec";
+
   default:
     break;
   }
@@ -834,6 +901,10 @@ get_type_name (GtkType type, gboolean * is_pointer)
   if (g_type_is_a (type, G_TYPE_BOXED))
     *is_pointer = TRUE;
 
+  /* All pointer subtypes will be pointers as well. */
+  if (g_type_is_a (type, G_TYPE_POINTER))
+    *is_pointer = TRUE;
+
   return type_name;
 }
 
@@ -893,228 +964,231 @@ get_gdk_event (gchar * signal_name)
 }
 
 
-/* This returns argument names to use for some known GTK signals. */
-/* Note: Could possibly return the types to use as well, since several of
-   them could be better, e.g. "GtkOrientation" instead of "gint". */
-static gchar **
-lookup_signal_arg_names (gchar * type, gchar * signal_name)
+static void
+init_signal_hash (void)
 {
-  /* Each arg array starts with the object type name and the signal name,
-     and then signal arguments follow. Note that the spacing is hardcoded. */
-  static gchar *GbArgTable[][8] =
-  {
-    {"GtkCList", "select_row",
-     "gint             row",
-     "gint             column",
-     "GdkEvent        *event"},
-    {"GtkCList", "unselect_row",
-     "gint             row",
-     "gint             column",
-     "GdkEvent        *event"},
-    {"GtkCList", "click_column",
-     "gint             column"},
+  static const gchar *signal_data[][2] = {
+    /*
+     * GTK+ Signals.
+     */
+    { "GtkAccelGroup::accel_activate", "accelgroup,object,key,modifier" },
+    { "GtkAccelGroup::accel_changed", "accelgroup,key,modifier,closure" },
+    { "GtkCellRendererText::edited", "cellrenderertext,path,new_text" },
+    { "GtkCellRendererToggle::toggled", "cellrenderertoggle,path" },
+    { "GtkCList::click_column", "clist,column" },
+    { "GtkCList::extend_selection", "clist,scroll_type,position,auto_start_selection" },
+    { "GtkCList::resize_column", "clist,column,width" },
+    { "GtkCList::row_move", "clist,source_row,dest_row" },
+    { "GtkCList::scroll_horizontal", "clist,scroll_type,position" },
+    { "GtkCList::scroll_vertical", "clist,scroll_type,position" },
+    { "GtkCList::select_row", "clist,row,column,event" },
+    { "GtkCList::set_scroll_adjustments", "clist,hadjustment,vadjustment" },
+    { "GtkCList::unselect_row", "clist,row,column,event" },
+    { "GtkCTree::change_focus_row_expansion", "ctree,expansion" },
+    { "GtkCTree::tree_collapse", "ctree,node" },
+    { "GtkCTree::tree_expand", "ctree,node" },
+    { "GtkCTree::tree_move", "ctree,node,new_parent,new_sibling" },
+    { "GtkCTree::tree_select_row", "ctree,node,column" },
+    { "GtkCTree::tree_unselect_row", "ctree,node,column" },
+    { "GtkDialog::response", "dialog,response_id" },
+    { "GtkEditable::delete_text", "editable,start_pos,end_pos" },
+    { "GtkEditable::insert_text", "editable,new_text,new_text_length,position" },
+    { "GtkEntry::delete_from_cursor", "entry,type,count" },
+    { "GtkEntry::insert_at_cursor", "entry,string" },
+    { "GtkEntry::move_cursor", "entry,step,count,extend_selection" },
+    { "GtkEntry::populate_popup", "entry,menu" },
+    { "GtkInputDialog::disable_device", "inputdialog,device" },
+    { "GtkInputDialog::enable_device", "inputdialog,device" },
+    { "GtkIMContext::commit", "imcontext,string" },
+    { "GtkIMContext::delete_surrounding", "imcontext,offset,n_chars" },
+    { "GtkLabel::move_cursor", "label,step,count,extend_selection" },
+    { "GtkLabel::populate_popup", "label,menu" },
+    { "GtkLayout::set_scroll_adjustments", "layout,hadjustment,vadjustment" },
+    { "GtkListItem::extend_selection", "listitem,scroll_type,position,auto_start_selection" },
+    { "GtkListItem::scroll_horizontal", "listitem,scroll_type,position" },
+    { "GtkListItem::scroll_vertical", "listitem,scroll_type,position" },
+    { "GtkMenuItem::toggle_size_allocate", "menuitem,allocation" },
+    { "GtkMenuItem::toggle_size_request", "menuitem,gint *requisition" },
+    { "GtkMenuShell::activate_current", "menushell,force_hide" },
+    { "GtkMenuShell::cycle_focus", "menushell,direction" },
+    { "GtkMenuShell::move_current", "menushell,direction" },
+    { "GtkNotebook::change_current_page", "notebook,offset" },
+    { "GtkNotebook::focus_tab", "notebook,type" },
+    { "GtkNotebook::move_focus_out", "notebook,direction" },
+    { "GtkNotebook::select_page", "notebook,move_focus" },
+    { "GtkNotebook::switch_page", "notebook,GtkNotebookPage *page,page_num" },
+    { "GtkOldEditable::kill_char", "oldeditable,direction" },
+    { "GtkOldEditable::kill_line", "oldeditable,direction" },
+    { "GtkOldEditable::kill_word", "oldeditable,direction" },
+    { "GtkOldEditable::move_cursor", "oldeditable,x,y" },
+    { "GtkOldEditable::move_page", "oldeditable,x,y" },
+    { "GtkOldEditable::move_to_column", "oldeditable,column" },
+    { "GtkOldEditable::move_to_row", "oldeditable,row" },
+    { "GtkOldEditable::move_word", "oldeditable,offset" },
+    { "GtkOldEditable::set_editable", "oldeditable,is_editable" },
+    { "GtkPaned::cycle_child_focus", "paned,reverse" },
+    { "GtkPaned::cycle_handle_focus", "paned,reverse" },
+    { "GtkPaned::move_handle", "paned,scroll" },
+    { "GtkRange::adjust_bounds", "range,value" },
+    { "GtkRange::move_slider", "range,scroll" },
+    { "GtkScale::format_value", "scale,value" },
+    { "GtkScrolledWindow::move_focus_out", "scrolledwindow,direction" },
+    { "GtkScrolledWindow::scroll_child", "scrolledwindow,scroll,horizontal" },
+    { "GtkSpinButton::change_value", "spinbutton,scroll" },
+    { "GtkSpinButton::input", "spinbutton,gdouble *new_value" },
+    { "GtkStatusbar::text_popped", "statusbar,context_id,text" },
+    { "GtkStatusbar::text_pushed", "statusbar,context_id,text" },
+    { "GtkTextBuffer::apply_tag", "textbuffer,tag,start,end" },
+    { "GtkTextBuffer::delete_range", "textbuffer,start,end" },
+    { "GtkTextBuffer::insert_child_anchor", "textbuffer,iter,anchor" },
+    { "GtkTextBuffer::insert_pixbuf", "textbuffer,iter,pixbuf" },
+    { "GtkTextBuffer::insert_text", "textbuffer,iter,text,length" },
+    { "GtkTextBuffer::mark_deleted", "textbuffer,mark" },
+    { "GtkTextBuffer::mark_set", "textbuffer,iter,mark" },
+    { "GtkTextBuffer::remove_tag", "textbuffer,tag,start,end" },
+    { "GtkText::set_scroll_adjustments", "text,hadjustment,vadjustment" },
+    { "GtkTextTag::event", "texttag,object,event,iter" },
+    { "GtkTextTagTable::tag_added", "texttagtable,tag" },
+    { "GtkTextTagTable::tag_changed", "texttagtable,tag,size_changed" },
+    { "GtkTextTagTable::tag_removed", "texttagtable,tag" },
+    { "GtkTextView::delete_from_cursor", "textview,type,count" },
+    { "GtkTextView::insert_at_cursor", "textview,string" },
+    { "GtkTextView::move_cursor", "textview,step,count,extend_selection" },
+    { "GtkTextView::move_focus", "textview,direction" },
+    { "GtkTextView::page_horizontally", "textview,count,extend_selection" },
+    { "GtkTextView::populate_popup", "textview,menu" },
+    { "GtkTextView::set_scroll_adjustments", "textview,hadjustment,vadjustment" },
+    { "GtkTipsQuery::widget_entered", "tipsquery,widget,tip_text,tip_private" },
+    { "GtkTipsQuery::widget_selected", "tipsquery,widget,tip_text,tip_private,event" },
+    { "GtkToolbar::orientation_changed", "toolbar,orientation" },
+    { "GtkToolbar::style_changed", "toolbar,style" },
+    { "GtkTreeModel::row_changed", "treemodel,path,iter" },
+    { "GtkTreeModel::row_deleted", "treemodel,path" },
+    { "GtkTreeModel::row_has_child_toggled", "treemodel,path,iter" },
+    { "GtkTreeModel::row_inserted", "treemodel,path,iter" },
+    { "GtkTreeModel::rows_reordered", "treemodel,path,iter,gint *new_order" },
+    { "GtkTreeView::expand_collapse_cursor_row", "treeview,logical,expand,open_all" },
+    { "GtkTreeView::move_cursor", "treeview,step,count" },
+    { "GtkTreeView::row_activated", "treeview,path,column" },
+    { "GtkTreeView::row_collapsed", "treeview,iter,path" },
+    { "GtkTreeView::row_expanded", "treeview,iter,path" },
+    { "GtkTreeView::select_cursor_row", "treeview,start_editing" },
+    { "GtkTreeView::set_scroll_adjustments", "treeview,hadjustment,vadjustment" },
+    { "GtkTreeView::test_collapse_row", "treeview,iter,path" },
+    { "GtkTreeView::test_expand_row", "treeview,iter,path" },
+    { "GtkViewport::set_scroll_adjustments", "viewport,hadjustment,vadjustment" },
+    { "GtkWidget::child_notify", "widget,pspec" },
+    { "GtkWidget::direction_changed", "widget,old_direction" },
+    { "GtkWidget::drag_begin", "widget,drag_context" },
+    { "GtkWidget::drag_data_delete", "widget,drag_context" },
+    { "GtkWidget::drag_data_get", "widget,drag_context,data,info,time" },
+    { "GtkWidget::drag_data_received", "widget,drag_context,x,y,data,info,time" },
+    { "GtkWidget::drag_drop", "widget,drag_context,x,y,time" },
+    { "GtkWidget::drag_end", "widget,drag_context" },
+    { "GtkWidget::drag_leave", "widget,drag_context,time" },
+    { "GtkWidget::drag_motion", "widget,drag_context,x,y,time" },
+    { "GtkWidget::focus", "widget,direction" },
+    { "GtkWidget::grab_notify", "widget,was_grabbed" },
+    { "GtkWidget::hierarchy_changed", "widget,previous_toplevel" },
+    { "GtkWidget::mnemonic_activate", "widget,group_cycling" },
+    { "GtkWidget::parent_set", "widget,old_parent" },
+    { "GtkWidget::selection_get", "widget,data,info,time" },
+    { "GtkWidget::selection_received", "widget,data,time" },
+    { "GtkWidget::show_help", "widget,help_type" },
+    { "GtkWidget::size_allocate", "widget,allocation" },
+    { "GtkWidget::size_request", "widget,requisition" },
+    { "GtkWidget::state_changed", "widget,state" },
+    { "GtkWidget::style_set", "widget,previous_style" },
+    { "GtkWindow::move_focus", "window,direction" },
 
-    {"GtkCList", "resize_column",
-     "gint             column",
-     "gint             width"},
+    /*
+     * libgnomeui Signals.
+     */
+    { "GnomeClient::connect", "client,restarted" },
+    { "GnomeClient::save_yourself", "client,phase,save_style,shutdown,interact_style,fast" },
+    { "GnomeColorPicker::color_set", "colorpicker,red,green,blue,alpha" },
+    { "GnomeDialog::clicked", "dialog,button_number" },
+    { "GnomeFontPicker::font_set", "fontpicker,font_name" },
+    { "GnomeIconList::focus_icon", "iconlist,num" },
+    { "GnomeIconList::move_cursor", "iconlist,direction,clear_selection" },
+    { "GnomeIconList::select_icon", "iconlist,num,event" },
+    { "GnomeIconList::text_changed", "iconlist,num,new_text" },
+    { "GnomeIconList::unselect_icon", "iconlist,num,event" },
+    { "GnomeMDI::add_child", "mdi,child" },
+    { "GnomeMDI::app_created", "mdi,app" },
+    { "GnomeMDI::child_changed", "mdi,child" },
+    { "GnomeMDI::remove_child", "mdi,child" },
+    { "GnomePropertyBox::apply", "propertybox,page_num" },
+    { "GnomePropertyBox::help", "propertybox,page_num" },
 
-    {"GtkCList", "extend_selection",
-     "GtkScrollType    scroll_type",
-     "gfloat           position",
-     "gboolean         auto_start_selection"},
-    {"GtkCList", "scroll_vertical",
-     "GtkScrollType    scroll_type",
-     "gfloat           position"},
-    {"GtkCList", "scroll_horizontal",
-     "GtkScrollType    scroll_type",
-     "gfloat           position"},
-    {"GtkContainer", "focus",
-     "GtkDirectionType direction"},
-    {"GtkCTree", "tree_select_row",
-     "GList           *node",
-     "gint             column"},
-    {"GtkCTree", "tree_unselect_row",
-     "GList           *node",
-     "gint             column"},
+    /*
+     * libgnomedb Signals.
+     */
+    { "GnomeDbBrowser::progress_message", "browser,message" },
+    { "GnomeDbDsnConfigDruid::finished", "druid,error" },
+    { "GnomeDbForm::model_changed", "form" },
+    { "GnomeDbGrid::row_selected", "grid,row" },
+    { "GnomeDbGrid::selection_cleared", "grid" },
+    { "GnomeDbList::row_selected", "list,row" },
+    { "GnomeDbList::selection_cleared", "list" },
 
-    {"GtkCTree", "tree_expand",
-     "GList           *node"},
-    {"GtkCTree", "tree_collapse",
-     "GList           *node"},
-    {"GtkCTree", "tree_move",
-     "GList           *node",
-     "GList           *new_parent",
-     "GList           *new_sibling"},
-    {"GtkCTree", "change_focus_row_expansion",
-     "GtkCTreeExpansionType expansion"},
-
-    {"GtkEditable", "insert_text",
-     "gchar           *new_text",
-     "gint             new_text_length",
-     "gint            *position"},
-    {"GtkEditable", "delete_text",
-     "gint             start_pos",
-     "gint             end_pos"},
-    {"GtkEditable", "set_editable",
-     "gboolean         is_editable"},
-    {"GtkEditable", "move_cursor",
-     "gint             x",
-     "gint             y"},
-    {"GtkEditable", "move_word",
-     "gint             num_words"},
-    {"GtkEditable", "move_page",
-     "gint             x",
-     "gint             y"},
-    {"GtkEditable", "move_to_row",
-     "gint             row"},
-    {"GtkEditable", "move_to_column",
-     "gint             column"},
-
-    {"GtkEditable", "kill_char",
-     "gint             direction"},
-    {"GtkEditable", "kill_word",
-     "gint             direction"},
-    {"GtkEditable", "kill_line",
-     "gint             direction"},
-
-
-    {"GtkInputDialog", "enable_device",
-     "gint             deviceid"},
-    {"GtkInputDialog", "disable_device",
-     "gint             deviceid"},
-
-    {"GtkListItem", "extend_selection",
-     "GtkScrollType    scroll_type",
-     "gfloat           position",
-     "gboolean         auto_start_selection"},
-    {"GtkListItem", "scroll_vertical",
-     "GtkScrollType    scroll_type",
-     "gfloat           position"},
-    {"GtkListItem", "scroll_horizontal",
-     "GtkScrollType    scroll_type",
-     "gfloat           position"},
-
-    {"GtkMenuShell", "move_current",
-     "GtkMenuDirectionType direction"},
-    {"GtkMenuShell", "activate_current",
-     "gboolean         force_hide"},
-
-
-    {"GtkNotebook", "switch_page",
-     "GtkNotebookPage *page",
-     "gint             page_num"},
-    {"GtkStatusbar", "text_pushed",
-     "guint            context_id",
-     "gchar           *text"},
-    {"GtkStatusbar", "text_popped",
-     "guint            context_id",
-     "gchar           *text"},
-    {"GtkTipsQuery", "widget_entered",
-     "GtkWidget       *widget",
-     "gchar           *tip_text",
-     "gchar           *tip_private"},
-    {"GtkTipsQuery", "widget_selected",
-     "GtkWidget       *widget",
-     "gchar           *tip_text",
-     "gchar           *tip_private",
-     "GdkEventButton  *event"},
-    {"GtkToolbar", "orientation_changed",
-     "GtkOrientation   orientation"},
-    {"GtkToolbar", "style_changed",
-     "GtkToolbarStyle  style"},
-    {"GtkWidget", "draw",
-     "GdkRectangle    *area"},
-    {"GtkWidget", "size_request",
-     "GtkRequisition  *requisition"},
-    {"GtkWidget", "size_allocate",
-     "GtkAllocation   *allocation"},
-    {"GtkWidget", "state_changed",
-     "GtkStateType     state"},
-    {"GtkWidget", "style_set",
-     "GtkStyle        *previous_style"},
-
-    {"GtkWidget", "install_accelerator",
-     "gchar           *signal_name",
-     "gchar            key",
-     "gint             modifiers"},
-
-    {"GtkWidget", "add_accelerator",
-     "guint            accel_signal_id",
-     "GtkAccelGroup   *accel_group",
-     "guint            accel_key",
-     "GdkModifierType  accel_mods",
-     "GtkAccelFlags    accel_flags"},
-
-    {"GtkWidget", "parent_set",
-     "GtkObject       *old_parent"},
-
-    {"GtkWidget", "remove_accelerator",
-     "GtkAccelGroup   *accel_group",
-     "guint            accel_key",
-     "GdkModifierType  accel_mods"},
-
-    {"GtkWidget", "debug_msg",
-     "gchar           *message"},
-    {"GtkWindow", "move_resize",
-     "gint            *x",
-     "gint            *y",
-     "gint             width",
-     "gint             height"},
-    {"GtkWindow", "set_focus",
-     "GtkWidget       *widget"},
-
-    {"GtkWidget", "selection_get",
-     "GtkSelectionData *data",
-     "guint            info",
-     "guint            time"},
-    {"GtkWidget", "selection_received",
-     "GtkSelectionData *data",
-     "guint            time"},
-
-    {"GtkWidget", "drag_begin",
-     "GdkDragContext  *drag_context"},
-    {"GtkWidget", "drag_end",
-     "GdkDragContext  *drag_context"},
-    {"GtkWidget", "drag_data_delete",
-     "GdkDragContext  *drag_context"},
-    {"GtkWidget", "drag_leave",
-     "GdkDragContext  *drag_context",
-     "guint            time"},
-    {"GtkWidget", "drag_motion",
-     "GdkDragContext  *drag_context",
-     "gint             x",
-     "gint             y",
-     "guint            time"},
-    {"GtkWidget", "drag_drop",
-     "GdkDragContext  *drag_context",
-     "gint             x",
-     "gint             y",
-     "guint            time"},
-    {"GtkWidget", "drag_data_get",
-     "GdkDragContext  *drag_context",
-     "GtkSelectionData *data",
-     "guint            info",
-     "guint            time"},
-    {"GtkWidget", "drag_data_received",
-     "GdkDragContext  *drag_context",
-     "gint             x",
-     "gint             y",
-     "GtkSelectionData *data",
-     "guint            info",
-     "guint            time"},
-
-    {NULL}
+    { NULL, NULL }
   };
 
   gint i;
 
-  for (i = 0; GbArgTable[i][0]; i++)
+  glade_signal_hash = g_hash_table_new (g_str_hash, g_str_equal);
+
+  for (i = 0; signal_data[i][0]; i++)
     {
-      if (!strcmp (type, GbArgTable[i][0])
-	  && !strcmp (signal_name, GbArgTable[i][1]))
-	return &GbArgTable[i][2];
+      g_hash_table_insert (glade_signal_hash, (char*) signal_data[i][0],
+			   (char*) signal_data[i][1]);
     }
-  return NULL;
+}
+
+
+/* This returns argument names to use for some known signals.
+   The returned array must be freed with g_str_freev(). */
+static gchar **
+lookup_signal_arg_names (gchar * type, gchar * signal_name,
+			 gint num_args_expected)
+{
+  char *signal_id, *arg_names;
+  char **arg_array = NULL;
+
+  if (glade_signal_hash == NULL)
+    init_signal_hash ();
+
+  signal_id = g_strdup_printf ("%s::%s", type, signal_name);
+  arg_names = g_hash_table_lookup (glade_signal_hash, signal_id);
+#if 0
+  g_print ("Found signal: %s args: %s\n", signal_id, arg_names);
+#endif
+
+  if (arg_names)
+    {
+      gint num_args = 0;
+
+      arg_array = g_strsplit (arg_names, ",", -1);
+
+      while (arg_array[num_args])
+	num_args++;
+
+      /* Check we got the expected number of parameters. If not, output a
+	 warning and return NULL. */
+      if (num_args != num_args_expected)
+	{
+	  g_warning ("Internal error: argument names invalid for signal: %s."
+		     "Expected %i arguments. Found %i",
+		     signal_id, num_args_expected, num_args);
+	  g_strfreev (arg_array);
+	  arg_array = NULL;
+	}
+    }
+
+  g_free (signal_id);
+
+  return arg_array;
 }
 
 
