@@ -27,6 +27,7 @@
 #include "glade.h"
 #include "styles.h"
 #include "glade_widget_data.h"
+#include "glade-parser.h"
 #include "gbsource.h"
 
 #ifdef __cplusplus
@@ -50,9 +51,18 @@ extern "C" {
 #define GLADE_IS_CUSTOM_WIDGET(w) \
 	(gtk_object_get_data(GTK_OBJECT(w), GLADE_CUSTOM_KEY) != NULL)
 
+/* This is used to save a pointer to the GladeWidgetInfo inside a widget while
+   loading, so we can resolve ATK relations afterwards. */
+extern const gchar *GladeWidgetInfoKey;
 
-extern const gchar *GladeMenuItemStockIndexKey;
+extern const gchar *GladeMenuItemStockIDKey;
 extern const gchar *GladeMenuItemIconKey;
+extern const gchar *GladeMenuItemStockIndexKey;
+
+/* This is the key we use to store the stock icon name or full pathname.
+   It is used for GtkImage (hence in the menu editor), and also for the
+   icons in buttons. */
+extern const gchar *GladeIconKey;
 
 
 /*************************************************************************
@@ -137,8 +147,16 @@ struct _GbWidgetGetArgData
   GladeProject  *project;
   GbWidgetAction action;		/* GB_SHOWING or GB_SAVING. */
 
+  GtkWidget	*widget;		/* The widget being processed. Note
+					   that this is also passed in as an
+					   argument, but having it in the
+					   struct as well helps with
+					   translatable properties. */
   GladeWidgetData *widget_data;		/* The GladeWidgetData of the widget
 					   currently being processed. */
+  gchar		*agent;			/* This is set to the agent parameter
+					   when outputting properties for C++
+					   etc., e.g. "glademm". */
 
   /* These are only used when saving (to a file or the clipboard). */
   gboolean	 copying_to_clipboard;	/* TRUE if we are copying the widgets
@@ -161,40 +179,6 @@ struct _GbWidgetGetArgData
 };
 
 
-typedef enum
-{
-  GB_TOKEN_START_TAG,
-  GB_TOKEN_END_TAG,
-  GB_TOKEN_DATA
-} GbTokenType;
-
-typedef struct _GbLoadedProperty  GbLoadedProperty;
-struct _GbLoadedProperty
-{
-  gint tag_index;
-  gint cdata_index;
-  gint line_number;
-};
-
-struct _GbLoadPropControl
-{
-  GbLoadedProperty *ptr;  /* pointer to property */
-  gint		 n;       /* number of buffers   */
-  gint		 space;   /* max buffer space    */
-};
-typedef struct _GbLoadPropControl GbLoadPropControl;
-
-#define GB_LOADED_PROPERTIES_INC 16
-
-/* FIXME: Use GString everywhere instead. */
-struct _GbBuffControl
-{
-  gchar      	*ptr;     /* pointer to buffer   */
-  gint		 pos;     /* position in buffer  */
-  gint		 space;   /* max buffer space    */
-};
-typedef struct _GbBuffControl GbBuffControl;
-
 /* Each widget has two types of properties, standard properties which are all
    the properties shown on the 'Widget' and 'Basic' pages in the property
    editor, and child properties, shown on the 'Place' page, which relate to
@@ -213,36 +197,45 @@ typedef enum
 struct _GbWidgetSetArgData {
   GladeProject      *project;
   GbWidgetAction     action;		/* GB_APPLYING or GB_LOADING. */
+  GtkWidget	    *widget;		/* The widget being processed. Note
+					   that this is also passed in as an
+					   argument, but having it in the
+					   struct as well helps with
+					   translatable properties. */
   GladeWidgetData   *widget_data;
   GtkWidget	    *property_to_apply;
   gboolean	     apply;
 
   /* These are only used when loading. If we are loading from a file, filename
-   and fp are used. If we are pasting from the clipboard, xml_buffer_*,
-   replacing_widget and discard_names are used. */
+     is used. If we are pasting from the clipboard, xml_buffer,
+     replacing_widget and discard_names are used. */
   gchar		    *filename;
-  FILE		    *fp;
 
   gchar		    *xml_buffer;
-  gint		     xml_buffer_pos;
-  gint		     xml_buffer_len;
   GtkWidget         *replacing_widget;
   gboolean	     discard_names;
 
-  gint		     line_number;
-  GbBuffControl	     buffer;
-  GbTokenType	     token_type;
-  gint		     token;
   GladeStatusCode    status;
-  GbLoadPropControl  properties;
-  GbLoadPropControl  child_properties;  
+
+  GladeInterface    *interface;
+
+  /* This is the GladeChildInfo & GladeWidgetInfo of the widget currently
+     being created. */
+  GladeChildInfo    *child_info;
+  GladeWidgetInfo   *widget_info;
+
+  /* As we load widgets, we add them all to this hash, so that the ATK code
+     can find the targets of relations in a second pass. The key is the
+     widget name, and the value is the widget. */
+  GHashTable	    *all_widgets;
+
+  /* This tells the load_XXX() functions to use the normal or packing
+     properties. */
   GbPropertyType     loading_type;
-  GList		    *signals;
-  GList		    *accelerators;
-#ifdef GLADE_STYLE_SUPPORT
-  GbStyle	    *gbstyle;
-#endif
-  GList		    *error_messages;
+
+  /* This is set to the agent we want to match when loading properties,
+     e.g. "glademm" for the C++ properties. */
+  gchar		    *agent;
 };
 
 
@@ -285,6 +278,10 @@ struct _GbWidget
   gint		  properties_page_number;	/* Used internally */
   gint		  child_properties_page_number;	/* Used internally */
 
+  /* We need a pixbuf rather than GdkPixmap for the tree. This is only
+     created when needed for now. */
+  GdkPixbuf	 *pixbuf;
+
   /* Creates a new widget of the particular class, or possibly shows a dialog
      for the user to set options before creating the widget (but it must not
      do this when loading an XML file, i.e. when the action is GB_LOADING). */
@@ -450,10 +447,14 @@ gchar*	    gb_widget_get_child_name	(GtkWidget	       *widget);
 
 /* Sets the child name of the widget. The child_name string is duplicated. */
 void	    gb_widget_set_child_name	(GtkWidget	       *widget,
-					 gchar		       *child_name);
+					 const gchar	       *child_name);
 
 
-/* Inputting Properties - Loading or Applying. */
+/* Inputting Properties - Loading or Applying.
+   Note: The only time you have to free the returned value is when calling
+   gb_widget_input_text() and data->action == GB_APPLYING (because we get the
+   value from the text widget and it has to be freed), and when calling
+   gb_widget_input_pixmap_filename() with data->action == GB_LOADING. */
 gchar*	    gb_widget_input_string	(GbWidgetSetArgData    *data,
 					 const gchar	       *property);
 gchar*	    gb_widget_input_text	(GbWidgetSetArgData    *data,
@@ -485,7 +486,17 @@ gchar*	    gb_widget_input_pixmap_filename (GbWidgetSetArgData *data,
 GdkFont*    gb_widget_input_font	(GbWidgetSetArgData    *data,
 					 const gchar	       *property,
 					 gchar		      **xlfd_fontname);
+gchar*	    gb_widget_input_stock_item	(GbWidgetSetArgData	*data,
+					 const gchar		*property);
+gchar*	    gb_widget_input_icon	(GbWidgetSetArgData	*data,
+					 const gchar		*property);
 
+/* load only */
+gint        gb_widget_input_enum        (GbWidgetSetArgData     *data,
+					 GType                   enum_type,
+					 const char            **labels,
+					 int                    *values,
+					 const gchar            *property);
 
 /* Outputting Properties - Saving or Showing.
    When saving, the translatable string and text properties may also be output
@@ -549,6 +560,20 @@ void	    gb_widget_output_font	(GbWidgetGetArgData    *data,
 					 const gchar	       *property,
 					 GdkFont	       *value,
 					 const gchar	       *xlfd_fontname);
+void	    gb_widget_output_stock_item (GbWidgetGetArgData	*data,
+					 const gchar		*property,
+					 const gchar		*value);
+void	    gb_widget_output_icon	(GbWidgetGetArgData	*data,
+					 const gchar		*property,
+					 const gchar		*value);
+
+void        gb_widget_output_enum       (GbWidgetGetArgData     *data,
+					 GType                   enum_type,
+					 int                    *values,
+					 int                     n_values,
+					 const gchar            *property,
+					 gint                    value);
+
 
 
 /* Tooltips functions. */
@@ -559,10 +584,12 @@ void	    gb_widget_reset_tooltips	(void);
 /* Convenience functions for handling all 6 properties of adjustments. */
 void	    gb_widget_output_adjustment	(GbWidgetGetArgData *data,
 					 const gchar	    *Values[],
-					 GtkAdjustment	    *adjustment);
+					 GtkAdjustment	    *adjustment,
+					 gchar		    *saved_property_name);
 gboolean    gb_widget_input_adjustment	(GbWidgetSetArgData *data, 
 					 const gchar	    *Values[],
-					 GtkAdjustment	    *adjustment);
+					 GtkAdjustment	    *adjustment,
+					 gchar		    *saved_property_name);
 
 
 /* Common functions for gbwidgets that have a child label - button/menuitem
@@ -586,9 +613,6 @@ void	    gb_widget_children_foreach	(GtkWidget	       *widget,
 					 GtkCallback		callback,
 					 gpointer		data);
 
-void        gb_init_load_properties (GbLoadPropControl *p);
-void        gb_free_load_properties (GbLoadPropControl *p);
-void        gb_init_buffer_struct (int pos, int space, GbBuffControl * p);
 
 #ifdef __cplusplus
 }
