@@ -54,25 +54,6 @@ static const gchar *GbDirectionSymbols[] =
 };
 
 
-static const gchar *GbShadowChoices[] =
-{ "None", "In", "Out", "Etched In", "Etched Out", NULL};
-static const gint GbShadowValues[] =
-{
-  GTK_SHADOW_NONE,
-  GTK_SHADOW_IN,
-  GTK_SHADOW_OUT,
-  GTK_SHADOW_ETCHED_IN,
-  GTK_SHADOW_ETCHED_OUT
-};
-static const gchar *GbShadowSymbols[] =
-{
-  "GTK_SHADOW_NONE",
-  "GTK_SHADOW_IN",
-  "GTK_SHADOW_OUT",
-  "GTK_SHADOW_ETCHED_IN",
-  "GTK_SHADOW_ETCHED_OUT"
-};
-
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
  * to the function in the GbWidget struct at the end of this file.
@@ -104,7 +85,7 @@ gb_arrow_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
   property_add_choice (Direction, _("Direction:"), _("The direction of the arrow"),
 		       GbDirectionChoices);
   property_add_choice (Shadow, _("Shadow:"), _("The shadow type of the arrow"),
-		       GbShadowChoices);
+		       GladeShadowChoices);
   property_add_float_range (XAlign, _("X Align:"),
 			    _("The horizontal alignment of the arrow"),
 			    0, 1, 0.01, 0.1, 0.01, 2);
@@ -135,10 +116,10 @@ gb_arrow_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 	gb_widget_output_choice (data, Direction, i, GbDirectionSymbols[i]);
     }
 
-  for (i = 0; i < sizeof (GbShadowValues) / sizeof (GbShadowValues[0]); i++)
+  for (i = 0; i < GladeShadowChoicesSize; i++)
     {
-      if (GbShadowValues[i] == GTK_ARROW (widget)->shadow_type)
-	gb_widget_output_choice (data, Shadow, i, GbShadowSymbols[i]);
+      if (GladeShadowValues[i] == GTK_ARROW (widget)->shadow_type)
+	gb_widget_output_choice (data, Shadow, i, GladeShadowSymbols[i]);
     }
   gb_widget_output_float (data, XAlign, GTK_MISC (widget)->xalign);
   gb_widget_output_float (data, YAlign, GTK_MISC (widget)->yalign);
@@ -159,8 +140,8 @@ gb_arrow_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
   gboolean set_arrow = FALSE, set_alignment = FALSE, set_padding = FALSE;
   gfloat xalign, yalign;
   gint xpad, ypad, i;
-  GtkArrowType arrow_type;
-  GtkShadowType shadow_type;
+  GtkArrowType arrow_type = GTK_ARROW_UP;
+  GtkShadowType shadow_type = GTK_SHADOW_NONE;
 
   direction = gb_widget_input_choice (data, Direction);
   if (data->apply)
@@ -184,14 +165,12 @@ gb_arrow_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
   shadow = gb_widget_input_choice (data, Shadow);
   if (data->apply)
     {
-      for (i = 0;
-	   i < sizeof (GbShadowValues) / sizeof (GbShadowValues[0]);
-	   i++)
+      for (i = 0; i < GladeShadowChoicesSize; i++)
 	{
-	  if (!strcmp (shadow, GbShadowChoices[i])
-	      || !strcmp (shadow, GbShadowSymbols[i]))
+	  if (!strcmp (shadow, GladeShadowChoices[i])
+	      || !strcmp (shadow, GladeShadowSymbols[i]))
 	    {
-	      shadow_type = GbShadowValues[i];
+	      shadow_type = GladeShadowValues[i];
 	      set_arrow = TRUE;
 	      break;
 	    }
@@ -269,8 +248,8 @@ gb_arrow_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 	arrow_type = i;
 	break;
       }
-  for (i = 0; i < sizeof (GbShadowValues) / sizeof (GbShadowValues[0]); i++)
-    if (GbShadowValues[i] == GTK_ARROW (widget)->shadow_type)
+  for (i = 0; i < GladeShadowChoicesSize; i++)
+    if (GladeShadowValues[i] == GTK_ARROW (widget)->shadow_type)
       {
 	shadow_type = i;
 	break;
@@ -278,7 +257,7 @@ gb_arrow_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
   if (data->create_widget)
     {
       source_add (data, "  %s = gtk_arrow_new (%s, %s);\n", data->wname,
-	      GbDirectionSymbols[arrow_type], GbShadowSymbols[shadow_type]);
+	      GbDirectionSymbols[arrow_type], GladeShadowSymbols[shadow_type]);
     }
   gb_widget_write_standard_source (widget, data);
 
@@ -309,7 +288,8 @@ GbWidget *
 gb_arrow_init ()
 {
   /* Initialise the GTK type */
-  gtk_arrow_get_type ();
+  volatile GtkType type;
+  type = gtk_arrow_get_type ();
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct (&gbwidget);
