@@ -28,13 +28,9 @@
  */
 static GbWidget gbwidget;
 
-/* These are the special children of the widget. */
-static gchar *InputDialogSaveButton = "InputDialog:save_button";
-static gchar *InputDialogCloseButton = "InputDialog:close_button";
-
 static gchar *Title = "InputDialog|GtkWindow::title";
 static gchar *Type = "InputDialog|GtkWindow::type";
-static gchar *Position = "InputDialog|GtkWindow::position";
+static gchar *Position = "InputDialog|GtkWindow::window_position";
 static gchar *Modal = "InputDialog|GtkWindow::modal";
 static gchar *DefaultWidth = "Input|GtkWindow::default_width";
 static gchar *DefaultHeight = "Input|GtkWindow::default_height";
@@ -44,6 +40,16 @@ static gchar *AutoShrink = "InputDialog|GtkWindow::auto_shrink";
 static gchar *WMName = "InputDialog|GtkWindow::wmclass_name";
 static gchar *WMClass = "InputDialog|GtkWindow::wmclass_class";
 
+static gchar *Resizable = "InputDialog|GtkWindow::resizable";
+static gchar *DestroyWithParent = "InputDialog|GtkWindow::destroy_with_parent";
+static gchar *Icon = "InputDialog|GtkWindow::icon";
+
+static gchar *Role = "InputDialog|GtkWindow::role";
+static gchar *TypeHint = "InputDialog|GtkWindow::type_hint";
+static gchar *SkipTaskbar = "InputDialog|GtkWindow::skip_taskbar_hint";
+static gchar *SkipPager = "InputDialog|GtkWindow::skip_pager_hint";
+static gchar *Decorated = "InputDialog|GtkWindow::decorated";
+static gchar *Gravity = "InputDialog|GtkWindow::gravity";
 
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
@@ -64,18 +70,25 @@ gb_input_dialog_new (GbWidgetNewData *data)
 
   GtkInputDialog *inputdlg = GTK_INPUT_DIALOG (new_widget);
 
+  /* We want it to be treated as a normal window. */
+  gtk_window_set_type_hint (GTK_WINDOW (new_widget),
+			    GDK_WINDOW_TYPE_HINT_NORMAL);
+
   gtk_signal_connect (GTK_OBJECT (new_widget), "delete_event",
 		      GTK_SIGNAL_FUNC (editor_close_window), NULL);
 
   gb_widget_create_from (inputdlg->save_button,
 			 data->action == GB_CREATING ? "save_button" : NULL);
-  gb_widget_set_child_name (inputdlg->save_button, InputDialogSaveButton);
+  gb_widget_set_child_name (inputdlg->save_button, GladeChildSaveButton);
   /* We need to set it sensitive so the user can select it. */
   gtk_widget_set_sensitive (inputdlg->save_button, TRUE);
 
   gb_widget_create_from (inputdlg->close_button,
 			 data->action == GB_CREATING ? "close_button" : NULL);
-  gb_widget_set_child_name (inputdlg->close_button, InputDialogCloseButton);
+  gb_widget_set_child_name (inputdlg->close_button, GladeChildCloseButton);
+
+  gtk_object_set_data (GTK_OBJECT (new_widget), TypeHint,
+		       GINT_TO_POINTER (GLADE_TYPE_HINT_DIALOG_INDEX));
 
   return new_widget;
 }
@@ -92,7 +105,10 @@ gb_input_dialog_create_properties (GtkWidget * widget, GbWidgetCreateArgData * d
 					Title, Type, Position, Modal,
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, AutoShrink,
-					WMName, WMClass);
+					WMName, WMClass,
+					Resizable, DestroyWithParent, Icon,
+					Role, TypeHint, SkipTaskbar,
+					SkipPager, Decorated, Gravity);
 }
 
 
@@ -108,7 +124,10 @@ gb_input_dialog_get_properties (GtkWidget *widget, GbWidgetGetArgData * data)
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 }
 
 
@@ -124,7 +143,10 @@ gb_input_dialog_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 }
 
 
@@ -152,6 +174,8 @@ gb_input_dialog_create_popup_menu (GtkWidget * widget, GbWidgetCreateMenuData *
 static void
 gb_input_dialog_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 {
+  gchar *wname, *child_name;
+
   if (data->create_widget)
     {
       source_add (data, "  %s = gtk_input_dialog_new ();\n", data->wname);
@@ -163,7 +187,39 @@ gb_input_dialog_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data
 				   Title, Type, Position, Modal,
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
-				   WMName, WMClass);
+				   WMName, WMClass,
+				   Resizable, DestroyWithParent, Icon,
+				   Role, TypeHint, SkipTaskbar,
+				   SkipPager, Decorated, Gravity);
+
+  /* We output the source code for the buttons here, but we don't want them
+     to be created. We need to remember the dialog's name since data->wname
+     will be overwritten. */
+  wname = g_strdup (data->wname);
+
+  source_add (data, "\n");
+
+  child_name = (gchar*) gtk_widget_get_name (GTK_INPUT_DIALOG (widget)->save_button);
+  child_name = source_create_valid_identifier (child_name);
+  source_add (data, "  %s = GTK_INPUT_DIALOG (%s)->save_button;\n",
+	      child_name, wname);
+  g_free (child_name);
+  data->create_widget = FALSE;
+  gb_widget_write_source (GTK_INPUT_DIALOG (widget)->save_button,
+			  data);
+
+  child_name = (gchar*) gtk_widget_get_name (GTK_INPUT_DIALOG (widget)->close_button);
+  child_name = source_create_valid_identifier (child_name);
+  source_add (data, "  %s = GTK_INPUT_DIALOG (%s)->close_button;\n",
+	      child_name, wname);
+  g_free (child_name);
+  data->create_widget = FALSE;
+  gb_widget_write_source (GTK_INPUT_DIALOG (widget)->close_button,
+			  data);
+
+  g_free (wname);
+
+  data->write_children = FALSE;
 }
 
 
@@ -171,9 +227,9 @@ static GtkWidget *
 gb_input_dialog_get_child (GtkWidget * widget,
 			   const gchar * child_name)
 {
-  if (!strcmp (child_name, InputDialogSaveButton))
+  if (!strcmp (child_name, GladeChildSaveButton))
     return GTK_INPUT_DIALOG (widget)->save_button;
-  else if (!strcmp (child_name, InputDialogCloseButton))
+  else if (!strcmp (child_name, GladeChildCloseButton))
     return GTK_INPUT_DIALOG (widget)->close_button;
   else
     return NULL;
@@ -189,7 +245,8 @@ GbWidget*
 gb_input_dialog_init ()
 {
   /* Initialise the GTK type */
-  gtk_input_dialog_get_type();
+  volatile GtkType type;
+  type = gtk_input_dialog_get_type();
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct(&gbwidget);
@@ -205,6 +262,7 @@ gb_input_dialog_init ()
   gbwidget.gb_widget_get_properties	= gb_input_dialog_get_properties;
   gbwidget.gb_widget_set_properties	= gb_input_dialog_set_properties;
   gbwidget.gb_widget_write_source = gb_input_dialog_write_source;
+  gbwidget.gb_widget_destroy = gb_window_destroy;
 /*
   gbwidget.gb_widget_create_popup_menu	= gb_input_dialog_create_popup_menu;
 */
