@@ -32,7 +32,7 @@ static gchar *ClimbRate = "GtkSpinButton::climb_rate";
 static gchar *Digits = "GtkSpinButton::digits";
 static gchar *Numeric = "GtkSpinButton::numeric";
 static gchar *Policy = "GtkSpinButton::update_policy";
-static gchar *Snap = "GtkSpinButton::snap";
+static gchar *Snap = "GtkSpinButton::snap_to_ticks";
 static gchar *Wrap = "GtkSpinButton::wrap";
 
 
@@ -45,16 +45,6 @@ static const gchar *Values[] =
   "GtkSpinButton::page",
   "GtkSpinButton::page_size",
 };
-/* These are for backwards compatability, for pre Glade 0.5.2. */
-static const gchar *OldValues[] =
-{
-  "GtkSpinButton::hvalue",
-  "GtkSpinButton::hlower",
-  "GtkSpinButton::hupper",
-  "GtkSpinButton::hstep",
-  "GtkSpinButton::hpage",
-  "GtkSpinButton::hpage_size",
-};
 
 static const gchar *GbPolicyChoices[] =
 {"Always", "If Valid", NULL};
@@ -104,7 +94,7 @@ gb_spin_button_create_properties (GtkWidget * widget, GbWidgetCreateArgData *
 			  _("The number of decimal digits to show"),
 			  0, 5, 1, 1, 0);
   property_add_bool (Numeric, _("Numeric:"),
-		     _("If any value entered should be converted according to the locale settings"));
+		     _("If only numeric entry is allowed"));
   property_add_choice (Policy, _("Update Policy:"),
 		       _("When value_changed signals are emitted"),
 		       GbPolicyChoices);
@@ -142,7 +132,9 @@ gb_spin_button_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
   gb_widget_output_bool (data, Snap, GTK_SPIN_BUTTON (widget)->snap_to_ticks);
 
   gb_widget_output_bool (data, Wrap, GTK_SPIN_BUTTON (widget)->wrap);
-  gb_widget_output_adjustment (data, Values, GTK_SPIN_BUTTON (widget)->adjustment);
+  gb_widget_output_adjustment (data, Values,
+			       GTK_SPIN_BUTTON (widget)->adjustment,
+			       "adjustment");
 }
 
 
@@ -194,11 +186,8 @@ gb_spin_button_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
   if (data->apply)
     gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (widget), wrap);
 
-  /* If the new property names aren't used, try the old ones when loading. */
   adj = GTK_SPIN_BUTTON (widget)->adjustment;
-  if (gb_widget_input_adjustment (data, Values, adj)
-      || (data->action == GB_LOADING
-	  && gb_widget_input_adjustment (data, OldValues, adj)))
+  if (gb_widget_input_adjustment (data, Values, adj, "adjustment"))
     {
       gtk_signal_emit_by_name (GTK_OBJECT (adj), "value_changed");
     }
@@ -236,11 +225,11 @@ gb_spin_button_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
     {
       source_add_decl (data, "  GtkObject *%s_adj;\n", data->real_wname);
       source_add (data,
-		"  %s_adj = gtk_adjustment_new (%g, %g, %g, %g, %g, %g);\n",
+		"  %s_adj = gtk_adjustment_new (%.12g, %.12g, %.12g, %.12g, %.12g, %.12g);\n",
 		  data->real_wname, adj->value, adj->lower, adj->upper,
 		  adj->step_increment, adj->page_increment, adj->page_size);
       source_add (data,
-		  "  %s = gtk_spin_button_new (GTK_ADJUSTMENT (%s_adj), %g, %d);\n",
+		  "  %s = gtk_spin_button_new (GTK_ADJUSTMENT (%s_adj), %.12g, %d);\n",
 		  data->wname, data->real_wname,
 		  GTK_SPIN_BUTTON (widget)->climb_rate,
 		  GTK_SPIN_BUTTON (widget)->digits);
@@ -284,7 +273,8 @@ GbWidget *
 gb_spin_button_init ()
 {
   /* Initialise the GTK type */
-  gtk_spin_button_get_type ();
+  volatile GtkType type;
+  type = gtk_spin_button_get_type ();
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct (&gbwidget);
