@@ -36,6 +36,8 @@
 #include "load.h"
 #include "tree.h"
 #include "utils.h"
+#include "gb.h"
+#include "glade_project_window.h"
 
 typedef struct _GladeClipboardItem  GladeClipboardItem;
 
@@ -71,10 +73,10 @@ static void glade_clipboard_on_project_destroy (GladeProject *project,
 #define GLADE_PASTE_BUFFER_INCREMENT	1024
 
 
-guint
+GType
 glade_clipboard_get_type (void)
 {
-  static guint glade_clipboard_type = 0;
+  static GType glade_clipboard_type = 0;
 
   if (!glade_clipboard_type)
     {
@@ -108,19 +110,17 @@ glade_clipboard_class_init (GladeClipboardClass * klass)
 }
 
 
-/*
- * FIXME: If this is supposed to be a dialog, it should be transient for
- * something. Otherwise, it can't be iconized.
- */
 static void
 glade_clipboard_init (GladeClipboard * clipboard)
 {
   GtkWidget *vbox, *scrolled_win;
 
-  GTK_WINDOW (clipboard)->type = GTK_WINDOW_DIALOG;
   gtk_window_set_title (GTK_WINDOW (clipboard), _ ("Clipboard"));
   gtk_window_set_policy (GTK_WINDOW (clipboard), FALSE, TRUE, FALSE);
   gtk_window_set_wmclass (GTK_WINDOW (clipboard), "clipboard", "Glade");
+  gtk_window_set_default_size (GTK_WINDOW (clipboard), 150, 200);
+  gtk_window_add_accel_group (GTK_WINDOW (clipboard),
+			      glade_get_global_accel_group ());
 
   vbox = gtk_vbox_new (FALSE, 4);
   gtk_widget_show (vbox);
@@ -144,6 +144,10 @@ glade_clipboard_init (GladeClipboard * clipboard)
   gtk_signal_connect (GTK_OBJECT (clipboard), "delete_event",
 		      GTK_SIGNAL_FUNC (glade_util_close_window_on_delete),
 		      NULL);
+
+  gtk_signal_connect_after (GTK_OBJECT (clipboard), "hide",
+                      GTK_SIGNAL_FUNC (glade_project_window_uncheck_clipboard_menu_item),
+                      NULL);
 }
 
 
@@ -178,9 +182,10 @@ glade_clipboard_cut_or_copy (GladeClipboard *clipboard,
 			     GtkWidget      *widget,
 			     gboolean	     cut)
 {
-  GbWidgetGetArgData data;
+  GbWidgetGetArgData data = { 0 };
   gchar *old_locale, *saved_locale;
   GList *selection;
+  gboolean is_component;
 
   if (widget == NULL)
     {
@@ -209,13 +214,41 @@ glade_clipboard_cut_or_copy (GladeClipboard *clipboard,
   saved_locale = g_strdup (old_locale);
   setlocale (LC_NUMERIC, "C");
 
+  g_string_append (data.buffer,
+		   "<?xml version=\"1.0\" standalone=\"no\"?> <!--*- mode: xml -*-->\n"
+		   "<!DOCTYPE glade-interface SYSTEM \"http://glade.gnome.org/glade-project-2.0.dtd\">\n"
+		   "\n"
+		   "<glade-interface>\n");
+
+  /* If the widget is not a toplevel widget, we add a dummy parent to it.
+     This is so that we keep the packing properties of the widget. */
+  is_component = glade_util_is_component (widget);
+  if (!is_component)
+    {
+      g_string_append (data.buffer,
+		       "<widget class=\"GtkWindow\" id=\"glade-dummy-container\">\n");
+      data.indent++;
+    }
+
   gb_widget_save (widget, &data);
 
+  if (!is_component)
+    {
+      g_string_append (data.buffer,
+		       "</widget>\n");
+    }
+
+  g_string_append (data.buffer,
+		   "\n</glade-interface>\n");
+
   setlocale (LC_NUMERIC, saved_locale);
   g_free (saved_locale);
 
   if (data.error == NULL)
     {
+#if 0
+      g_print ("Adding to clipboard:\n%s\n", data.buffer->str);
+#endif
       glade_clipboard_add (GLADE_CLIPBOARD (clipboard), project, cut,
 			   widget, data.buffer->str);
       if (cut)
@@ -229,16 +262,70 @@ glade_clipboard_cut_or_copy (GladeClipboard *clipboard,
 }
 
 
+static void
+initialize_all_widgets_cb (GtkWidget *widget, GbWidgetSetArgData *data)
+{
+  const gchar *widget_name;
+
+  /* If this is the widget we are replacing, just return. We don't want to
+     add it or its descendants to all_widgets. They will no longer exist, so
+     ATK relations can't use them. */
+  if (widget == data->replacing_widget)
+    return;
+
+  /* Add this widget to the all_widgets hash. */
+  if (GB_IS_GB_WIDGET (widget))
+    {
+      widget_name = gtk_widget_get_name (widget);
+      if (widget_name && *widget_name)
+	{
+#if 0
+	  g_print ("Initializing pointer to widget: %s, %p\n",
+		   widget_name, widget);
+#endif
+	  g_hash_table_insert (data->all_widgets, (gpointer) widget_name,
+			       widget);
+	}
+    }
+
+  /* Now recursively add the children. */
+  gb_widget_children_foreach (widget, (GtkCallback) initialize_all_widgets_cb,
+			      data);
+}
+
+
+/* This adds all widgets that are in the window being pasted into to the
+   all_widgets hash, so ATK relations to other widgets will be set when
+   pasting. */
+static void
+glade_clipboard_initialize_all_widgets (GbWidgetSetArgData *data)
+{
+  GtkWidget *toplevel;
+
+  /* If we aren't pasting into a component, just return. */
+  if (!data->replacing_widget)
+    return;
+
+  toplevel = glade_util_get_toplevel (data->replacing_widget);
+
+  initialize_all_widgets_cb (toplevel, data);
+}
+
+
+/* The widget argument is the widget to be pasted over, i.e. the widget in
+   the current interface that will be replaced by the clipboard item.
+   If it is NULL, the currently selected widget will be used. */
 void
 glade_clipboard_paste (GladeClipboard *clipboard,
 		       GladeProject   *project,
 		       GtkWidget      *widget)
 {
   GladeClipboardItem *item;
-  GbWidgetSetArgData data;
+  GbWidgetSetArgData data = { 0 };
   GtkWidget *parent, *new_widget = NULL;
-  gchar *old_locale, *saved_locale, *saved_timezone, *child_name;
+  gchar *saved_locale, *saved_timezone, *child_name;
   GList *selection;
+  gboolean is_component = FALSE;
 
   item = glade_clipboard_get_current_item (clipboard);
   if (item == NULL)
@@ -251,21 +338,23 @@ glade_clipboard_paste (GladeClipboard *clipboard,
 	widget = GTK_WIDGET (selection->data);
     }
 
+  if (gtk_type_is_a (item->type, gtk_window_get_type ())
+      || gtk_type_is_a (item->type, gtk_menu_get_type ()))
+    is_component = TRUE;
+
   /* We can only paste toplevel components, i.e. windows and menus, into the
      project, and not into other widgets. */
   if (widget == NULL)
     {
-      if (!gtk_type_is_a (item->type, gtk_window_get_type ())
-	  && !gtk_type_is_a (item->type, gtk_menu_get_type ()))
+      if (!is_component)
 	{
-	  glade_util_show_message_box (_("You need to select a widget to paste into"));
+	  glade_util_show_message_box (_("You need to select a widget to paste into"), widget);
 	  return;
 	}
     }
   else
     {
-      if (gtk_type_is_a (item->type, gtk_window_get_type ())
-	  || gtk_type_is_a (item->type, gtk_menu_get_type ()))
+      if (is_component)
 	widget = NULL;
     }
 
@@ -284,7 +373,7 @@ glade_clipboard_paste (GladeClipboard *clipboard,
 #endif
 	  )
 	{
-	  glade_util_show_message_box (_("You can't paste into windows or dialogs."));
+	  glade_util_show_message_box (_("You can't paste into windows or dialogs."), widget);
 	  return;
 	}
 
@@ -292,24 +381,22 @@ glade_clipboard_paste (GladeClipboard *clipboard,
       child_name = gb_widget_get_child_name (widget);
       if (child_name)
 	{
-	  if (!strcmp (child_name, "FileSel:ok_button")
-	      || !strcmp (child_name, "FileSel:cancel_button")
-	      || !strcmp (child_name, "ColorSel:ok_button")
-	      || !strcmp (child_name, "ColorSel:cancel_button")
-	      || !strcmp (child_name, "ColorSel:help_button")
-	      || !strcmp (child_name, "FontSel:ok_button")
-	      || !strcmp (child_name, "FontSel:cancel_button")
-	      || !strcmp (child_name, "FontSel:apply_button")
-	      || !strcmp (child_name, "InputDialog:save_button")
-	      || !strcmp (child_name, "InputDialog:close_button")
-
-	      || !strcmp (child_name, "Dialog:vbox")
-	      || !strcmp (child_name, "Dialog:action_area")
-	      || !strcmp (child_name, "GnomeDialog:vbox")
-	      || !strcmp (child_name, "GnomeDialog:action_area")
+	  if (!strcmp (child_name, GladeChildOKButton)
+	      || !strcmp (child_name, GladeChildCancelButton)
+	      || !strcmp (child_name, GladeChildHelpButton)
+	      || !strcmp (child_name, GladeChildApplyButton)
+	      || !strcmp (child_name, GladeChildSaveButton)
+	      || !strcmp (child_name, GladeChildCloseButton)
+	      || !strcmp (child_name, GladeChildDialogVBox)
+	      || !strcmp (child_name, GladeChildDialogActionArea)
+	      || !strcmp (child_name, GladeChildComboEntry)
+	      || !strcmp (child_name, GladeChildComboList)
+	      || !strcmp (child_name, GladeChildFontSelection)
+	      || !strcmp (child_name, GladeChildColorSelection)
+	      || !strcmp (child_name, GladeChildGnomeEntry)
 	      )
 	    {
-	      glade_util_show_message_box (_("You can't paste into the selected widget, since\nit is created automatically by its parent."));
+	      glade_util_show_message_box (_("You can't paste into the selected widget, since\nit is created automatically by its parent."), widget);
 	      return;
 	    }
 	}
@@ -318,7 +405,7 @@ glade_clipboard_paste (GladeClipboard *clipboard,
       if (GTK_IS_MENU_SHELL (widget)
 	  && !gtk_type_is_a (item->type, gtk_menu_item_get_type ()))
 	{
-	  glade_util_show_message_box (_("Only menu items can be pasted into a menu or menu bar."));
+	  glade_util_show_message_box (_("Only menu items can be pasted into a menu or menu bar."), widget);
 	  return;
 	}
 
@@ -326,48 +413,48 @@ glade_clipboard_paste (GladeClipboard *clipboard,
       if (GTK_IS_MENU_ITEM (widget)
 	  && !gtk_type_is_a (item->type, gtk_menu_item_get_type ()))
 	{
-	  glade_util_show_message_box (_("Only menu items can be pasted into a menu or menu bar."));
+	  glade_util_show_message_box (_("Only menu items can be pasted into a menu or menu bar."), widget);
 	  return;
 	}
 
-      /* Only buttons can be pasted into a GnomeDialog action area. */
+      /* Only buttons can be pasted into a dialog action area. */
       if (widget->parent)
 	{
 	  child_name = gb_widget_get_child_name (widget->parent);
-	  if (child_name && !strcmp (child_name, "GnomeDialog:action_area")
-	      && !gtk_type_is_a (item->type, gtk_button_get_type ()))
+	  if (child_name && !strcmp (child_name, GladeChildDialogActionArea)
+	      && item->type != GTK_TYPE_BUTTON)
 	    {
-	      glade_util_show_message_box (_("Only buttons can be pasted into a GnomeDialog action area."));
+	      glade_util_show_message_box (_("Only buttons can be pasted into a dialog action area."), widget);
 	      return;
 	    }
 	}
 
 #ifdef USE_GNOME
-      if (GNOME_IS_DOCK (widget))
+      if (BONOBO_IS_DOCK (widget))
 	{
-	  if (!gtk_type_is_a (item->type, gnome_dock_item_get_type ()))
+	  if (!gtk_type_is_a (item->type, bonobo_dock_item_get_type ()))
 	    {
-	      glade_util_show_message_box (_("Only GnomeDockItem widgets can be pasted into a GnomeDock."));
+	      glade_util_show_message_box (_("Only GnomeDockItem widgets can be pasted into a GnomeDock."), widget);
 	      return;
 	    }
 	}
 
-      if (GNOME_IS_DOCK_ITEM (widget))
+      if (BONOBO_IS_DOCK_ITEM (widget))
 	{
-	  if (!gtk_type_is_a (item->type, gnome_dock_item_get_type ()))
+	  if (!gtk_type_is_a (item->type, bonobo_dock_item_get_type ()))
 	    {
-	      glade_util_show_message_box (_("Only GnomeDockItem widgets can be pasted over a GnomeDockItem."));
+	      glade_util_show_message_box (_("Only GnomeDockItem widgets can be pasted over a GnomeDockItem."), widget);
 	      return;
 	    }
-	  glade_util_show_message_box (_("Sorry - pasting over a GnomeDockItem is not implemented yet."));
+	  glade_util_show_message_box (_("Sorry - pasting over a GnomeDockItem is not implemented yet."), widget);
 	  return;
 	}
 
-      if (gtk_type_is_a (item->type, gnome_dock_item_get_type ()))
+      if (gtk_type_is_a (item->type, bonobo_dock_item_get_type ()))
 	{
-	  if (!GNOME_IS_DOCK (widget) && !GNOME_IS_DOCK_ITEM (widget))
+	  if (!BONOBO_IS_DOCK (widget) && !BONOBO_IS_DOCK_ITEM (widget))
 	    {
-	      glade_util_show_message_box (_("GnomeDockItem widgets can only be pasted into a GnomeDock."));
+	      glade_util_show_message_box (_("GnomeDockItem widgets can only be pasted into a GnomeDock."), widget);
 	      return;
 	    }
 	}
@@ -375,11 +462,10 @@ glade_clipboard_paste (GladeClipboard *clipboard,
     }
 
   data.project = project;
-  data.fp = NULL;
   data.filename = NULL;
   data.xml_buffer = item->xml_data;
-  data.xml_buffer_pos = 0;
-  data.xml_buffer_len = strlen (data.xml_buffer);
+  data.status = GLADE_STATUS_OK;
+  data.all_widgets = g_hash_table_new (g_str_hash, g_str_equal);
 
   /* We normally create new names for the widgets pasted, discarding the names
      in the XML data. However, if the widgets were cut to the clipboard, and
@@ -409,9 +495,8 @@ glade_clipboard_paste (GladeClipboard *clipboard,
     {
       if (GTK_IS_FIXED (widget)
 	  || GTK_IS_LAYOUT (widget)
-	  || GTK_IS_PACKER (widget)
 #ifdef USE_GNOME
-	  || GNOME_IS_DOCK (widget)
+	  || BONOBO_IS_DOCK (widget)
 #endif
 	  )
 	{
@@ -424,59 +509,71 @@ glade_clipboard_paste (GladeClipboard *clipboard,
 	}
     }
 
-  load_init_before_read (GLADE_PASTE_BUFFER_INCREMENT, &data);
-
-  old_locale = setlocale (LC_NUMERIC, NULL);
-  saved_locale = g_strdup (old_locale);
-  setlocale (LC_NUMERIC, "C");
+  /* Initialize the all_widgets hash with all widget names in the component,
+     except the widget being replaced and any of its descendants. This is
+     done so that ATK relations to targets in the component will still be OK.
+  */
+  glade_clipboard_initialize_all_widgets (&data);
 
   tree_freeze ();
 
-  /* Save the current time zone environment variable. */
-  saved_timezone = g_strdup (getenv ("TZ"));
-  if (putenv ("TZ=GMT") == -1)
-    data.status = GLADE_STATUS_ERROR;
-  else
+  /* Set the locale to "C". */
+  saved_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+  setlocale (LC_NUMERIC, "C");
+
+  /* Set the timezone to "UTC". */
+  saved_timezone = glade_util_set_timezone ("UTC");
+
+  /* Now parse the clipboard data. */
+  data.interface = glade_parser_parse_buffer (data.xml_buffer,
+					      strlen (data.xml_buffer), NULL);
+  if (!data.interface || data.interface->n_toplevels != 1)
     {
-      load_token_skip_whitespace (&data);
-      if (data.status == GLADE_STATUS_OK)
+      /* I don't think this should happen, except due to bugs. */
+      g_warning ("Error pasting from clipboard");
+      glade_interface_destroy (data.interface);
+      g_hash_table_destroy (data.all_widgets);
+      return;
+    }
+
+  /* If the widget is a toplevel component, we use the top widget info and
+     no child info. If not, we use the first child, as the toplevel will be
+     a dummy container. */
+  data.child_info = NULL;
+  data.widget_info = data.interface->toplevels[0];
+
+  if (!is_component)
+    {
+      if (data.widget_info->n_children != 1
+	  || data.widget_info->children == NULL)
 	{
-	  if (!strcmp (data.buffer.ptr + data.token, "widget"))
-	    new_widget = gb_widget_load (NULL, &data, parent);
-	  else
-	    data.status = GLADE_STATUS_INVALID_ENTITY;
+	  data.status = GLADE_STATUS_ERROR;
+	}
+      else
+	{
+	  data.child_info = &data.widget_info->children[0];
+	  data.widget_info = data.child_info->child;
 	}
     }
 
-  tree_thaw ();
+  /* Create the widget. */
+  if (data.status == GLADE_STATUS_OK)
+    new_widget = gb_widget_load (NULL, &data, parent);
 
-  g_free (data.buffer.ptr);
-  gb_free_load_properties (&data.properties);
-  gb_free_load_properties (&data.child_properties);
+  /* Destroy the parse data. */
+  glade_interface_destroy (data.interface);
 
-  /* Now reset the timezone. */
-  if (saved_timezone)
-    {
-      gchar *new_timezone = g_strdup_printf ("TZ=%s", saved_timezone);
-      putenv (new_timezone);
-      g_free (new_timezone);
-      g_free (saved_timezone);
-    }
-  else
-    {
-#ifdef HAVE_SETENV
-      unsetenv ("TZ");
-#else
-      /* This doesn't work on my system. We need to reset TZ here. The only
-	 other way I can think of doing this is by changing environ.
-	 Note that this code is needed in load.c & glade_clipboard.c */
-      putenv ("TZ=");
-#endif
-    }
+  /* Reset the timezone. */
+  glade_util_reset_timezone (saved_timezone);
 
+  /* Reset the locale. */
   setlocale (LC_NUMERIC, saved_locale);
   g_free (saved_locale);
 
+  tree_thaw ();
+
+  g_hash_table_destroy (data.all_widgets);
+
   if (data.status != GLADE_STATUS_OK)
     {
       /* I don't think this should happen, except due to bugs. */
@@ -507,7 +604,7 @@ glade_clipboard_add	(GladeClipboard *clipboard,
   GbWidget *gbwidget;
   gchar *name;
 
-  name = gtk_widget_get_name (widget);
+  name = (char*) gtk_widget_get_name (widget);
   gbwidget = gb_widget_lookup (widget);
   g_return_if_fail (gbwidget != NULL);
 
@@ -520,7 +617,7 @@ glade_clipboard_add	(GladeClipboard *clipboard,
   /* Connect to the project's destroy signal to set the pointer to NULL, so
      we never have invalid pointers. */
   gtk_signal_connect (GTK_OBJECT (project), "destroy",
-		      glade_clipboard_on_project_destroy,
+		      GTK_SIGNAL_FUNC (glade_clipboard_on_project_destroy),
 		      item);
 
   gtk_clist_insert (GTK_CLIST (clipboard->clist), 0, &name);
