@@ -21,7 +21,7 @@
 
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkcheckmenuitem.h>
-#include <gtk/gtkfilesel.h>
+#include <gtk/gtkfilechooserdialog.h>
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkimagemenuitem.h>
@@ -49,12 +49,31 @@
 #include "editor.h"
 #include "gbwidget.h"
 #include "load.h"
+#include "property.h"
+#include "tree.h"
 #include "utils.h"
 
 /* This is used to store a pointer to a GladeProjectWindow in the main window.
  */
 static gchar *GladeProjectWindowKey = "GladeProjectWindowKey";
 
+/* The menuitem's to show/hide the various windows. */
+static GtkWidget *palette_item = NULL;
+static GtkWidget *property_editor_item = NULL;
+static GtkWidget *widget_tree_item = NULL;
+static GtkWidget *clipboard_item = NULL;
+
+/* stuff for opening a file when dragged into glade */
+enum
+{
+  TARGET_URI_LIST
+};
+
+static GtkTargetEntry drop_types[] =
+{
+  {"text/uri-list", 0, TARGET_URI_LIST}
+};
+
 
 static gboolean glade_project_window_delete_event (GtkWidget *widget,
 						   GdkEvent *event,
@@ -68,7 +87,14 @@ static void glade_project_window_on_new_project_ok (GladeProjectWindow *project_
 static void glade_project_window_on_open_project (GtkWidget *widget,
 					       gpointer   data);
 static void glade_project_window_real_open_project (GtkWidget          *widget,
+						    gint		response_id,
 						    GladeProjectWindow *project_window);
+
+static void glade_project_window_drag_data_received (GtkWidget *widget,
+                                                     GdkDragContext *context,
+                                                     gint x, gint y,
+                                                     GtkSelectionData *selection_data,
+                                                     guint info, guint time, gpointer data);
 #if 0
 static void glade_project_window_show_loading_errors (GladeProjectWindow *project_window,
 						      GList		 *errors);
@@ -87,6 +113,7 @@ static void glade_project_window_on_save_project_as (GtkWidget *widget,
 						     gpointer   data);
 static void glade_project_window_save_project_as (GladeProjectWindow *project_window);
 static void glade_project_window_save_as_callback (GtkWidget          *widget,
+						   gint		   response_id,
 						   GladeProjectWindow *project_window);
 #endif
 static gboolean glade_project_window_real_save_project (GladeProjectWindow *project_window,
@@ -112,13 +139,13 @@ static void glade_project_window_delete (GtkWidget *widget,
 					 gpointer   data);
 static void glade_project_window_real_delete (GladeProjectWindow *project_window);
 
-static void glade_project_window_show_palette (GtkWidget *widget,
+static void glade_project_window_toggle_palette (GtkWidget *widget,
 					       gpointer   data);
-static void glade_project_window_show_property_editor (GtkWidget *widget,
+static void glade_project_window_toggle_property_editor (GtkWidget *widget,
 						       gpointer   data);
-static void glade_project_window_show_widget_tree (GtkWidget *widget,
+static void glade_project_window_toggle_widget_tree (GtkWidget *widget,
 						   gpointer   data);
-static void glade_project_window_show_clipboard (GtkWidget *widget,
+static void glade_project_window_toggle_clipboard (GtkWidget *widget,
 						 gpointer   data);
 static void glade_project_window_toggle_tooltips (GtkWidget *widget,
 						  gpointer   data);
@@ -144,6 +171,7 @@ static void glade_project_window_update_title (GladeProjectWindow *project_windo
 
 
 
+#ifdef USE_GNOME
 static void
 glade_project_window_show_help_doc (const gchar *docname,
 				    GtkWidget *transient_widget)
@@ -162,20 +190,13 @@ glade_project_window_show_help_doc (const gchar *docname,
 }
 
 
-static void
-glade_project_window_show_turbo_start (GtkWidget *menuitem,
-				       gpointer data)
-{
-  glade_project_window_show_help_doc ("glade-turbo-start", menuitem);
-}
-
-
 static void
 glade_project_window_show_glade_faq (GtkWidget *menuitem,
 				     gpointer data)
 {
   glade_project_window_show_help_doc ("glade-faq", menuitem);
 }
+#endif
 
 
 /*
@@ -196,7 +217,7 @@ static GnomeUIInfo FileMenu[] =
     N_("Output the project source code"),
     glade_project_window_write_source, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK, GTK_STOCK_CONVERT,
-    'W', GDK_CONTROL_MASK, NULL
+    'B', GDK_CONTROL_MASK, NULL
   },
   {
     GNOME_APP_UI_ITEM, N_("Op_tions..."),
@@ -206,7 +227,7 @@ static GnomeUIInfo FileMenu[] =
     0, 0, NULL
   },
   GNOMEUIINFO_SEPARATOR,
-  GNOMEUIINFO_MENU_EXIT_ITEM (glade_project_window_quit, NULL),
+  GNOMEUIINFO_MENU_QUIT_ITEM (glade_project_window_quit, NULL),
   GNOMEUIINFO_END
 };
 
@@ -222,32 +243,40 @@ static GnomeUIInfo EditMenu[] =
   GNOMEUIINFO_END
 };
 
+
+/* These must match the indices of the appropriate items in the settings
+   GnomeUIInfo array. We need them to set the initial state. */
+#define GLADE_PALETTE_ITEM 0
+#define GLADE_PROPERTY_EDITOR_ITEM 1
+#define GLADE_WIDGET_TREE_ITEM 2
+#define GLADE_CLIPBOARD_ITEM 3
+
 static GnomeUIInfo ViewMenu[] =
 {
   {
-    GNOME_APP_UI_ITEM, N_("Show _Palette"), N_("Show the palette of widgets"),
-    glade_project_window_show_palette, NULL, NULL,
+    GNOME_APP_UI_TOGGLEITEM, N_("Show _Palette"), N_("Show the palette of widgets"),
+    glade_project_window_toggle_palette, NULL, NULL,
     GNOME_APP_PIXMAP_NONE, NULL,
     0, 0, NULL
   },
   {
-    GNOME_APP_UI_ITEM, N_("Show Property _Editor"),
+    GNOME_APP_UI_TOGGLEITEM, N_("Show Property _Editor"),
     N_("Show the property editor"),
-    glade_project_window_show_property_editor, NULL, NULL,
+    glade_project_window_toggle_property_editor, NULL, NULL,
     GNOME_APP_PIXMAP_NONE, NULL,
     0, 0, NULL
   },
   {
-    GNOME_APP_UI_ITEM, N_("Show Widget _Tree"),
+    GNOME_APP_UI_TOGGLEITEM, N_("Show Widget _Tree"),
     N_("Show the widget tree"),
-    glade_project_window_show_widget_tree, NULL, NULL,
+    glade_project_window_toggle_widget_tree, NULL, NULL,
     GNOME_APP_PIXMAP_NONE, NULL,
     0, 0, NULL
   },
   {
-    GNOME_APP_UI_ITEM, N_("Show _Clipboard"),
+    GNOME_APP_UI_TOGGLEITEM, N_("Show _Clipboard"),
     N_("Show the clipboard"),
-    glade_project_window_show_clipboard, NULL, NULL,
+    glade_project_window_toggle_clipboard, NULL, NULL,
     GNOME_APP_PIXMAP_NONE, NULL,
     0, 0, NULL
   },
@@ -308,14 +337,7 @@ static GnomeUIInfo HelpMenu[] =
 {
   GNOMEUIINFO_HELP ("glade-user-guide"),
   {
-    GNOME_APP_UI_ITEM, N_("Turbo Start"),
-    N_("View the turbo start guide"),
-    glade_project_window_show_turbo_start, NULL, NULL,
-    GNOME_APP_PIXMAP_NONE, NULL,
-    0, 0, NULL
-  },
-  {
-    GNOME_APP_UI_ITEM, N_("FAQ"),
+    GNOME_APP_UI_ITEM, N_("_FAQ"),
     N_("View the Glade FAQ"),
     glade_project_window_show_glade_faq, NULL, NULL,
     GNOME_APP_PIXMAP_NONE, NULL,
@@ -340,14 +362,11 @@ static GnomeUIInfo MainMenu[] =
 
 static GnomeUIInfo ToolBar[] =
 {
-  /* I've taken this out to try to make the window smaller. */
-  /*
   {
     GNOME_APP_UI_ITEM, N_("New"), N_("New Project"), 
     glade_project_window_new_project, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_NEW, 0, 0, NULL
   },
-  */
   {
     GNOME_APP_UI_ITEM, N_("Open"), N_("Open Project"), 
     glade_project_window_on_open_project, NULL, NULL,
@@ -364,7 +383,6 @@ static GnomeUIInfo ToolBar[] =
     glade_project_window_on_edit_options, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK, GTK_STOCK_PROPERTIES, 0, 0, NULL
   },
-  GNOMEUIINFO_SEPARATOR,
   {
     GNOME_APP_UI_ITEM, N_("Build"), N_("Build the Source Code"), 
     glade_project_window_write_source, NULL, NULL,
@@ -391,19 +409,19 @@ glade_project_window_new (void)
   gtk_window_set_wmclass (GTK_WINDOW (project_window->window),
 			  "project", "Glade");
 
-#ifdef FIXME
-  /* We want all the keyboard accelerators from the menus in the default
-     accel_group (or they won't work within the interface windows), so we
-     change the app's accel_group. This is a slight kludge. */
-  GNOME_APP (project_window->window)->accel_group = gtk_accel_group_get_default ();
-#endif
+  /* We want all the keyboard accelerators from the menus to work anywhere in
+     the Glade GUI, including windows being designed. So we add the accel
+     group to every window. */
+  GNOME_APP (project_window->window)->accel_group = glade_get_global_accel_group ();
+  gtk_window_add_accel_group (GTK_WINDOW (project_window->window),
+			      glade_get_global_accel_group ());
 
   /* Save a pointer to the GladeProjectWindow, so we can find it in callbacks.
    */
   gtk_object_set_data (GTK_OBJECT (project_window->window),
 		       GladeProjectWindowKey, project_window);
 
-  /*gtk_widget_set_uposition (project_window->window, 0, 0);*/
+  gtk_window_move (GTK_WINDOW (project_window->window), 0, 0);
 
   gtk_signal_connect (GTK_OBJECT (project_window->window), "destroy",
 		      GTK_SIGNAL_FUNC (glade_project_window_destroy), NULL);
@@ -415,8 +433,23 @@ glade_project_window_new (void)
 			    GTK_SIGNAL_FUNC (glade_project_window_key_press_event),
 			    NULL);
 
+  /* support for opening a file by dragging onto the project window */
+  gtk_drag_dest_set (project_window->window,
+                     GTK_DEST_DEFAULT_ALL,
+                     drop_types, G_N_ELEMENTS (drop_types),
+                     GDK_ACTION_COPY | GDK_ACTION_MOVE);
+
+  g_signal_connect (G_OBJECT (project_window->window), "drag-data-received",
+                    G_CALLBACK (glade_project_window_drag_data_received), NULL);
+
   gnome_app_create_menus (GNOME_APP (project_window->window), MainMenu);
+
   /* Set the states of the toggle items. */
+  palette_item = ViewMenu[GLADE_PALETTE_ITEM].widget;
+  property_editor_item = ViewMenu[GLADE_PROPERTY_EDITOR_ITEM].widget;
+  widget_tree_item = ViewMenu[GLADE_WIDGET_TREE_ITEM].widget;
+  clipboard_item = ViewMenu[GLADE_CLIPBOARD_ITEM].widget;
+
   show_grid_item = SettingsMenu[GLADE_SHOW_GRID_ITEM].widget;
   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (show_grid_item),
 				 editor_get_show_grid ());
@@ -547,7 +580,7 @@ glade_project_window_new (void)
   gtk_object_set_data (GTK_OBJECT (project_window->window),
 		       GladeProjectWindowKey, project_window);
 
-  /*gtk_widget_set_uposition (project_window->window, 0, 0);*/
+  gtk_window_move (GTK_WINDOW (project_window->window), 0, 0);
 
   gtk_signal_connect (GTK_OBJECT (project_window->window), "destroy",
 		      GTK_SIGNAL_FUNC (glade_project_window_destroy), NULL);
@@ -566,7 +599,7 @@ glade_project_window_new (void)
   gtk_widget_show (vbox_main);
 
   /* Create accelerator table */
-  accel_group = gtk_accel_group_new ();
+  accel_group = glade_get_global_accel_group ();
   gtk_window_add_accel_group (GTK_WINDOW (project_window->window),
 			      accel_group);
 
@@ -590,7 +623,7 @@ glade_project_window_new (void)
 
   /* create File menu */
   menuitem = gtk_menu_item_new_with_mnemonic (_("_Project"));
-  gtk_menu_bar_append (GTK_MENU_BAR (menubar), menuitem);
+  gtk_container_add (GTK_CONTAINER (menubar), menuitem);
   gtk_widget_show (menuitem);
 
   menu = gtk_menu_new ();
@@ -626,7 +659,7 @@ glade_project_window_new (void)
 		      GTK_SIGNAL_FUNC (glade_project_window_write_source),
 		      NULL);
   gtk_widget_add_accelerator (menuitem, "activate", accel_group,
-			      'W', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
+			      'B', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
   gtk_container_add (GTK_CONTAINER (menu), menuitem);
   gtk_tooltips_set_tip (tooltips, menuitem,
 			_("Output the project source code"), NULL);
@@ -656,7 +689,7 @@ glade_project_window_new (void)
 
   /* Create Edit menu */
   menuitem = gtk_menu_item_new_with_mnemonic (_("_Edit"));
-  gtk_menu_bar_append (GTK_MENU_BAR (menubar), menuitem);
+  gtk_container_add (GTK_CONTAINER (menubar), menuitem);
   gtk_widget_show (menuitem);
 
   menu = gtk_menu_new ();
@@ -685,49 +718,57 @@ glade_project_window_new (void)
 
   /* Create View menu */
   menuitem = gtk_menu_item_new_with_mnemonic (_("_View"));
-  gtk_menu_bar_append (GTK_MENU_BAR (menubar), menuitem);
+  gtk_container_add (GTK_CONTAINER (menubar), menuitem);
   gtk_widget_show (menuitem);
 
   menu = gtk_menu_new ();
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menu);
 
-  menuitem = gtk_menu_item_new_with_mnemonic (_("Show _Palette"));
+  menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show _Palette"));
   gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-		      GTK_SIGNAL_FUNC (glade_project_window_show_palette),
+		      GTK_SIGNAL_FUNC (glade_project_window_toggle_palette),
 		      NULL);
   gtk_container_add (GTK_CONTAINER (menu), menuitem);
   gtk_tooltips_set_tip (tooltips, menuitem, _("Show the palette of widgets"),
 			NULL);
+  palette_item = menuitem;
   gtk_widget_show (menuitem);
 
-  menuitem = gtk_menu_item_new_with_mnemonic (_("Show P_roperties"));
+  menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show Property _Editor"));
   gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-		      GTK_SIGNAL_FUNC (glade_project_window_show_property_editor),
+		      GTK_SIGNAL_FUNC (glade_project_window_toggle_property_editor),
 		      NULL);
   gtk_container_add (GTK_CONTAINER (menu), menuitem);
   gtk_tooltips_set_tip (tooltips, menuitem,
-			_("Show the properties of the selected widget"), NULL);
+			_("Show the property editor"), NULL);
+  property_editor_item = menuitem;
   gtk_widget_show (menuitem);
 
-  menuitem = gtk_menu_item_new_with_mnemonic (_("Show _Widget Tree"));
+  menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show Widget _Tree"));
   gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-		      GTK_SIGNAL_FUNC (glade_project_window_show_widget_tree),
+		      GTK_SIGNAL_FUNC (glade_project_window_toggle_widget_tree),
 		      NULL);
   gtk_container_add (GTK_CONTAINER (menu), menuitem);
   gtk_tooltips_set_tip (tooltips, menuitem,
-			_("Show the project widget tree"), NULL);
+			_("Show the widget tree"), NULL);
+  widget_tree_item = menuitem;
   gtk_widget_show (menuitem);
 
-  menuitem = gtk_menu_item_new_with_mnemonic (_("Show _Clipboard"));
+  menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show _Clipboard"));
   gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-		      GTK_SIGNAL_FUNC (glade_project_window_show_clipboard),
+		      GTK_SIGNAL_FUNC (glade_project_window_toggle_clipboard),
 		      NULL);
   gtk_container_add (GTK_CONTAINER (menu), menuitem);
   gtk_tooltips_set_tip (tooltips, menuitem,
 			_("Show the clipboard"), NULL);
+  clipboard_item = menuitem;
   gtk_widget_show (menuitem);
 
-  menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show Widget _Tooltips"));
+  menuitem = gtk_separator_menu_item_new ();
+  gtk_container_add (GTK_CONTAINER (menu), menuitem);
+  gtk_widget_show (menuitem);
+
+  menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show _Widget Tooltips"));
   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem),
 				 gb_widget_get_show_tooltips ());
   gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
@@ -790,7 +831,7 @@ glade_project_window_new (void)
 
   /* Create Help menu */
   menuitem = gtk_menu_item_new_with_mnemonic (_("_Help"));
-  gtk_menu_bar_append (GTK_MENU_BAR (menubar), menuitem);
+  gtk_container_add (GTK_CONTAINER (menubar), menuitem);
   gtk_widget_show (menuitem);
 
   menu = gtk_menu_new ();
@@ -966,6 +1007,31 @@ glade_project_window_new_project (GtkWidget *widget,
 }
 
 
+/* This sets the palette sensitive if a project is open or insensitive if
+   not, and shows/hides the GNOME widgets as appropriate. */
+static void
+glade_project_window_setup_interface (GladeProjectWindow *project_window)
+{
+  gboolean gnome_support = TRUE;
+  gboolean gnome_db_support = TRUE;
+
+  /* Make sure the arrow is selected in the palette. */
+  glade_palette_reset_selection (GLADE_PALETTE (glade_palette), FALSE);
+
+  if (current_project)
+    {
+      gnome_support = glade_project_get_gnome_support (current_project);
+      gnome_db_support = glade_project_get_gnome_db_support (current_project);
+    }
+
+  glade_palette_set_show_gnome_widgets (GLADE_PALETTE (glade_palette),
+					gnome_support, gnome_db_support);
+
+  gtk_widget_set_sensitive (GTK_BIN (glade_palette)->child,
+			    current_project ? TRUE : FALSE);
+}
+
+
 static void
 glade_project_window_on_new_project_ok (GladeProjectWindow *project_window)
 {
@@ -1024,8 +1090,9 @@ glade_project_window_on_new_project_ok (GladeProjectWindow *project_window)
   gtk_statusbar_push (GTK_STATUSBAR (project_window->statusbar), 1,
 		      _("New project created."));
 
-  /* Make sure the palette is sensitive now. */
-  gtk_widget_set_sensitive (glade_palette, TRUE);
+  /* Make sure the palette is sensitive now and shows the appropriate
+     widgets. */
+  glade_project_window_setup_interface (project_window);
 }
 
 
@@ -1035,45 +1102,54 @@ glade_project_window_on_open_project (GtkWidget *widget,
 {
   GladeProjectWindow *project_window;
   GtkWidget *filesel;
+  GtkFileFilter *filter;
 
   project_window = get_glade_project_window (widget);
   g_return_if_fail (project_window != NULL);
 
-  filesel = gtk_file_selection_new (_("Open Project"));
+  filesel = gtk_file_chooser_dialog_new (_("Open Project"),
+					 GTK_WINDOW (project_window->window),
+					 GTK_FILE_CHOOSER_ACTION_OPEN,
+					 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					 GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+					 NULL);
+  gtk_dialog_set_default_response (GTK_DIALOG (filesel), GTK_RESPONSE_OK);
   gtk_window_set_position (GTK_WINDOW (filesel), GTK_WIN_POS_MOUSE);
-  gtk_window_set_wmclass (GTK_WINDOW (filesel), "open_project", "Glade");  
-  if (GTK_IS_WINDOW (project_window->window))
-    gtk_window_set_transient_for (GTK_WINDOW (filesel),
-				  GTK_WINDOW (project_window->window));
-  if (project_window->current_directory)
-    gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel),
-				     project_window->current_directory);
 
-  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
-		      "clicked",
-		      GTK_SIGNAL_FUNC (glade_project_window_real_open_project),
-		      project_window);
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
-			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (filesel));
-  gtk_signal_connect (GTK_OBJECT (filesel), "key_press_event",
-		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
-		      GINT_TO_POINTER (GladeEscDestroys));
+  g_signal_connect (filesel, "response",
+		    GTK_SIGNAL_FUNC (glade_project_window_real_open_project),
+		    project_window);
+
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (filter, "Glade Files");
+  gtk_file_filter_add_pattern (filter, "*.glade");
+  gtk_file_filter_add_pattern (filter, "*.glade2");
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (filesel), filter);
+
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (filter, "All Files");
+  gtk_file_filter_add_pattern (filter, "*");
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (filesel), filter);
+
+  if (project_window->current_directory)
+    glade_util_set_file_selection_filename (filesel, project_window->current_directory);
+
   gtk_widget_show (filesel);
 }
 
 
 static void
-glade_project_window_real_open_project (GtkWidget          *widget,
+glade_project_window_real_open_project (GtkWidget          *filesel,
+					gint		    response_id,
 					GladeProjectWindow *project_window)
 {
-  GtkWidget *filesel;
-  gchar *filename;
-
-  filesel = gtk_widget_get_toplevel (widget);
-  filename = (char*) gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
-
-  glade_project_window_open_project (project_window, filename);
+  if (response_id == GTK_RESPONSE_OK)
+    {
+      gchar *filename = glade_util_get_file_selection_filename (filesel);
+      if (filename)
+	glade_project_window_open_project (project_window, filename);
+      g_free (filename);
+    }
   gtk_widget_destroy (filesel);
 }
 
@@ -1085,6 +1161,10 @@ glade_project_window_open_project (GladeProjectWindow *project_window,
   GladeProject *project;
   gboolean status;
 
+  /* Clear the project view. Otherwise we may show an invalid project for
+     a while. */
+  glade_project_view_set_project (GLADE_PROJECT_VIEW (project_window->project_view), NULL);
+
   if (filename)
     {
       g_free (project_window->current_directory);
@@ -1099,8 +1179,6 @@ glade_project_window_open_project (GladeProjectWindow *project_window,
       glade_project_view_set_project (GLADE_PROJECT_VIEW (project_window->project_view), project);
       gtk_statusbar_push (GTK_STATUSBAR (project_window->statusbar), 1,
 			  _("Project opened."));
-      /* Make sure the palette is sensitive now. */
-      gtk_widget_set_sensitive (glade_palette, TRUE);
     }
   else
     {
@@ -1109,7 +1187,6 @@ glade_project_window_open_project (GladeProjectWindow *project_window,
 	 create a new empty GTK+ project. */
 #ifdef USE_GNOME
       project = NULL;
-      gtk_widget_set_sensitive (glade_palette, FALSE);
 #else
       project = glade_project_new ();
 #endif
@@ -1117,10 +1194,53 @@ glade_project_window_open_project (GladeProjectWindow *project_window,
       gtk_statusbar_push (GTK_STATUSBAR (project_window->statusbar), 1,
 			  _("Error opening project."));
     }
+
+  /* Make sure the palette is sensitive if needed and show the appropriate
+     widgets. */
+  glade_project_window_setup_interface (project_window);
+
   glade_project_window_update_title (project_window);
 }
 
 
+static void
+glade_project_window_drag_data_received (GtkWidget *widget,
+                                         GdkDragContext *context,
+                                         gint x, gint y,
+                                         GtkSelectionData *selection_data,
+                                         guint info, guint time, gpointer data)
+{
+  GladeProjectWindow *project_window;
+  gchar *uri_list;
+  GList *list = NULL;
+
+  if (info != TARGET_URI_LIST)
+    return;
+
+  /*
+   * Mmh... it looks like on Windows selection_data->data is not
+   * NULL terminated, so we need to make sure our local copy is.
+   */
+  uri_list = g_new (gchar, selection_data->length + 1);
+  memcpy (uri_list, selection_data->data, selection_data->length);
+  uri_list[selection_data->length] = 0;
+
+  project_window = get_glade_project_window (widget);
+  g_return_if_fail (project_window != NULL);
+
+  /* For now, if more than a file is dragged into glade we actually
+   * only try the first one... we can open only one anyway.
+   */
+  list = glade_util_uri_list_parse (uri_list);
+  if (list->data)
+    glade_project_window_open_project (project_window, list->data);
+
+  /* we can now free each string in the list */
+  g_list_foreach (list, (GFunc) g_free, NULL);
+  g_list_free (list);
+}
+
+
 /* This shows the errors in a dialog, and frees them. */
 /* Currently unused, since we switched to using libxml. */
 #if 0
@@ -1281,6 +1401,7 @@ glade_project_window_options_ok (GtkWidget	    *widget,
   action = options->action;
 
   glade_project_window_update_title (project_window);
+  glade_project_window_setup_interface (project_window);
 
   gtk_widget_destroy (GTK_WIDGET (options));
 
@@ -1351,50 +1472,50 @@ glade_project_window_save_project_as (GladeProjectWindow *project_window)
 {
   GtkWidget *filesel;
 
-  filesel = gtk_file_selection_new (_("Save Project"));
+  filesel = gtk_file_chooser_dialog_new (_("Save Project"),
+					 GTK_WINDOW (project_window->window),
+					 GTK_FILE_CHOOSER_ACTION_SAVE,
+					 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					 GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+					 NULL);
+  gtk_dialog_set_default_response (GTK_DIALOG (filesel), GTK_RESPONSE_OK);
   gtk_window_set_position (GTK_WINDOW (filesel), GTK_WIN_POS_MOUSE);
-  gtk_window_set_wmclass (GTK_WINDOW (filesel), "save_project", "Glade");  
-  if (GTK_IS_WINDOW (project_window->window))
-    gtk_window_set_transient_for (GTK_WINDOW (filesel),
-				  GTK_WINDOW (project_window->window));
+ 
+  g_signal_connect (options->filesel, "response",
+		    GTK_SIGNAL_FUNC (glade_project_window_save_as_callback),
+		    project_window);
+
   if (project_window->current_directory)
-    gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel),
-				     project_window->current_directory);
-  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
-		      "clicked",
-		      GTK_SIGNAL_FUNC (glade_project_window_save_as_callback),
-		      project_window);
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
-			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (filesel));
-  gtk_signal_connect (GTK_OBJECT (filesel), "key_press_event",
-		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
-		      GINT_TO_POINTER (GladeEscDestroys));
+    glade_util_set_file_selection_filename (filesel, project_window->current_directory);
+
   gtk_widget_show (filesel);
 }
 
 
 static void
-glade_project_window_save_as_callback (GtkWidget          *widget,
+glade_project_window_save_as_callback (GtkWidget          *filesel,
+				       gint		   response_id,
 				       GladeProjectWindow *project_window)
 {
   GladeProject *project;
-  GtkWidget *filesel;
   gchar *filename;
-  gboolean saved;
 
-  filesel = gtk_widget_get_toplevel (widget);
-  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
-
-  g_free (project_window->current_directory);
-  project_window->current_directory = glade_util_dirname (filename);
-
-  project = glade_project_view_get_project (GLADE_PROJECT_VIEW (project_window->project_view));
-  if (project)
+  if (response_id == GTK_RESPONSE_OK)
     {
-      glade_project_set_xml_filename (project, filename);
-      glade_project_window_real_save_project (project_window, TRUE);
+      filename = glade_util_get_file_selection_filename (filesel);
+
+      g_free (project_window->current_directory);
+      project_window->current_directory = glade_util_dirname (filename);
+
+      project = glade_project_view_get_project (GLADE_PROJECT_VIEW (project_window->project_view));
+      if (project)
+	{
+	  glade_project_set_xml_filename (project, filename);
+	  glade_project_window_real_save_project (project_window, TRUE);
+	}
+      g_free (filename);
     }
+
   gtk_widget_destroy (filesel);
 }
 #endif
@@ -1584,7 +1705,11 @@ glade_project_window_show_quit_dialog (GladeProjectWindow *project_window)
   gtk_window_set_wmclass (GTK_WINDOW (dialog), "quit", "Glade");
 
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
-    gtk_widget_destroy (project_window->window);
+    {
+      glade_save_settings (project_window, glade_palette, win_property,
+			   win_tree, glade_clipboard);
+      gtk_widget_destroy (project_window->window);
+    }
 
   gtk_widget_destroy (dialog);
 }
@@ -1672,34 +1797,107 @@ glade_project_window_real_delete (GladeProjectWindow *project_window)
 
 
 static void
-glade_project_window_show_palette (GtkWidget *widget,
-				   gpointer   data)
+refresh_menu_item (GtkWidget *menuitem, GtkWidget *window)
 {
-  glade_show_palette ();
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem),
+				  GTK_WIDGET_VISIBLE (window));
+}
+
+void
+glade_project_window_refresh_menu_items (void)
+{
+  refresh_menu_item (palette_item, glade_palette);
+  refresh_menu_item (property_editor_item, win_property);
+  refresh_menu_item (widget_tree_item, win_tree);
+  refresh_menu_item (clipboard_item, glade_clipboard);
+}
+
+
+void
+glade_project_window_uncheck_palette_menu_item (void)
+{
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (palette_item),
+				  FALSE);
+}
+
+
+void
+glade_project_window_uncheck_property_editor_menu_item (void)
+{
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (property_editor_item),
+				  FALSE);
+}
+
+
+void
+glade_project_window_uncheck_widget_tree_menu_item (void)
+{
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (widget_tree_item),
+				  FALSE);
+}
+
+
+void
+glade_project_window_uncheck_clipboard_menu_item (void)
+{
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (clipboard_item),
+				  FALSE);
 }
 
 
 static void
-glade_project_window_show_property_editor (GtkWidget *widget,
+glade_project_window_toggle_palette (GtkWidget *widget,
+				      gpointer   data)
+{
+  gboolean show_palette;
+
+  show_palette = GTK_CHECK_MENU_ITEM (widget)->active;
+  if (show_palette)
+    glade_show_palette ();
+  else
+    glade_hide_palette ();
+}
+
+
+static void
+glade_project_window_toggle_property_editor (GtkWidget *widget,
 					   gpointer   data)
 {
-  glade_show_property_editor ();
+  gboolean show_property_editor;
+
+  show_property_editor = GTK_CHECK_MENU_ITEM (widget)->active;
+  if (show_property_editor)
+    glade_show_property_editor ();
+  else
+    glade_hide_property_editor ();
 }
 
 
 static void
-glade_project_window_show_widget_tree (GtkWidget *widget,
+glade_project_window_toggle_widget_tree (GtkWidget *widget,
 				       gpointer   data)
 {
-  glade_show_widget_tree ();
+  gboolean show_widget_tree;
+
+  show_widget_tree =  GTK_CHECK_MENU_ITEM (widget)->active;
+  if (show_widget_tree)
+    glade_show_widget_tree ();
+  else
+    glade_hide_widget_tree ();
 }
 
 
 static void
-glade_project_window_show_clipboard (GtkWidget *widget,
+glade_project_window_toggle_clipboard (GtkWidget *widget,
 				     gpointer   data)
 {
-  glade_show_clipboard ();
+  gboolean show_clipboard;
+
+  show_clipboard =  GTK_CHECK_MENU_ITEM (widget)->active;
+  if (show_clipboard)
+    glade_show_clipboard ();
+  else
+    glade_hide_clipboard ();
 }
 
 
@@ -1759,87 +1957,97 @@ glade_project_window_about (GtkWidget *widget,
   /* VERSION comes from configure.in - the only place it should be defined */
 
 #ifdef USE_GNOME
-  const gchar *author[] = { "Damon Chaplin <damon@ximian.com>",
-			    "Martijn van Beers <martijn@earthling.net>",
-			    "Jacob Berkman <jacob@ximian.com>",
-			    NULL };
-  const gchar *documenter[] = { NULL };
-  GtkWidget *about, *transient_parent;
-  GdkPixbuf *pixbuf = NULL;
-  char *filename;
+  static GtkWidget *about = NULL;
 
-  filename = gnome_program_locate_file (gnome_program_get (),
-					GNOME_FILE_DOMAIN_APP_PIXMAP,
-					"glade-2.png", /* "glade-2/glade_logo.png", */
-					TRUE, NULL);
-  if (filename)
-	  pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+  if (about == NULL )
+    {
+      const gchar *author[] = { "Damon Chaplin <damon@gnome.org>",
+				"Martijn van Beers <martijn@earthling.net>",
+				"Jacob Berkman <jacob@ximian.com>",
+				NULL };
+      const gchar *documenter[] = { NULL };
+      GtkWidget *transient_parent;
+      GdkPixbuf *pixbuf = NULL;
+      char *filename;
 
-  about = gnome_about_new (_("Glade"),
-			   VERSION,
-			   _("(C) 1998-2002 Damon Chaplin"),
-			   _("Glade is a User Interface Builder for GTK+ and GNOME."),
-			   author, documenter, NULL, pixbuf);
+      filename = gnome_program_locate_file (gnome_program_get (),
+					    GNOME_FILE_DOMAIN_APP_PIXMAP,
+					    "glade-2.png", /* "glade-2/glade_logo.png", */
+					    TRUE, NULL);
+      if (filename)
+	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
 
-  if (pixbuf)
-	  gdk_pixbuf_unref (pixbuf);
+      about = gnome_about_new (_("Glade"),
+			       VERSION,
+			       _("(C) 1998-2002 Damon Chaplin"),
+			       _("Glade is a User Interface Builder for GTK+ and GNOME."),
+			       author, documenter, NULL, pixbuf);
+      if (pixbuf)
+	gdk_pixbuf_unref (pixbuf);
 			   
-  gtk_window_set_wmclass (GTK_WINDOW (about), "about", "Glade");
-  transient_parent = glade_util_get_toplevel (widget);
-  if (GTK_IS_WINDOW (transient_parent))
-    gtk_window_set_transient_for (GTK_WINDOW (about),
-				  GTK_WINDOW (transient_parent));
+      /* set the widget pointer to NULL when the widget is destroyed */
+      g_signal_connect (G_OBJECT (about), "destroy",
+			G_CALLBACK (gtk_widget_destroyed),
+			&about);
+      gtk_window_set_wmclass (GTK_WINDOW (about), "about", "Glade");
+      transient_parent = glade_util_get_toplevel (widget);
+      if (GTK_IS_WINDOW (transient_parent))
+	gtk_window_set_transient_for (GTK_WINDOW (about),
+				      GTK_WINDOW (transient_parent));
 
-#ifdef FIXME
-  hbox = gtk_hbox_new (TRUE, 0);
-  href = gnome_href_new ("http://glade.gnome.org/",
-			 _("Glade Home Page"));
-  gtk_box_pack_start (GTK_BOX (hbox), href, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (about)->vbox),
-		      hbox, TRUE, FALSE, 0);
-  gtk_widget_show_all (hbox);
-#endif
-
-  gtk_widget_show (about);
+      gtk_widget_show_all (GTK_WIDGET (about));
+    }
+  gtk_window_present (GTK_WINDOW (about));
 
 #else
-  GtkWidget *about, *vbox, *image, *label;
-  GtkWindow *transient_parent;
-  char *filename, *text;
+  static  GtkWidget *about = NULL;
 
-  transient_parent = GTK_WINDOW (glade_util_get_toplevel (widget));
-  about = gtk_dialog_new_with_buttons (_("About Glade"), transient_parent,
-				       0, GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
-  gtk_window_set_resizable (GTK_WINDOW (about), FALSE);
-  g_signal_connect (G_OBJECT (about), "response",
-		    G_CALLBACK (gtk_widget_destroy), NULL);
-  gtk_window_set_wmclass (GTK_WINDOW (about), "about", "Glade");
+  if (about == NULL )
+    {
+      GtkWidget *vbox, *image, *label;
+      GtkWindow *transient_parent;
+      char *filename, *text;
 
-  vbox = gtk_vbox_new (FALSE, 0);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 8);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about)->vbox), vbox, TRUE, TRUE, 0);
-  gtk_widget_show (vbox);
+      transient_parent = GTK_WINDOW (glade_util_get_toplevel (widget));
+      about = gtk_dialog_new_with_buttons (_("About Glade"), 
+					   transient_parent,
+					   0, 
+					   GTK_STOCK_OK, 
+					   GTK_RESPONSE_OK, 
+					   NULL);
+      gtk_window_set_resizable (GTK_WINDOW (about), FALSE);
+      g_signal_connect (G_OBJECT (about), "response",
+			G_CALLBACK (gtk_widget_destroy), NULL);
+      /* set the widget pointer to NULL when the widget is destroyed */
+      g_signal_connect (G_OBJECT (about), "destroy",
+			G_CALLBACK (gtk_widget_destroyed),
+			&about);
+      gtk_window_set_wmclass (GTK_WINDOW (about), "about", "Glade");
 
-  filename = g_strdup_printf ("%s/pixmaps/glade-2.png", GLADE_DATADIR);
-  image = gtk_image_new_from_file (filename);
-  g_free (filename);
-  gtk_widget_show (image);
-  gtk_box_pack_start (GTK_BOX (vbox), image, TRUE, TRUE, 8);
+      vbox = gtk_vbox_new (FALSE, 0);
+      gtk_container_set_border_width (GTK_CONTAINER (vbox), 8);
+      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about)->vbox), 
+			  vbox, TRUE, TRUE, 0);
 
-  text = g_strdup_printf ("<span size=\"xx-large\" weight=\"bold\">Glade "
+      filename = g_strdup_printf ("%s/pixmaps/glade-2.png", GLADE_DATADIR);
+      image = gtk_image_new_from_file (filename);
+      g_free (filename);
+      gtk_box_pack_start (GTK_BOX (vbox), image, TRUE, TRUE, 8);
+
+      text = g_strdup_printf ("<span size=\"xx-large\" weight=\"bold\">Glade "
 			  VERSION "</span>\n\n"
 			 "%s\n\n"
 			  "<span size=\"small\">%s</span>",
 			  _("Glade is a User Interface Builder for GTK+ and GNOME."),
 			  _("(C) 1998-2002 Damon Chaplin"));
 
-  label = gtk_label_new (text);
-  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
-  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
-  gtk_widget_show (label);
-
-  gtk_widget_show (about);
+      label = gtk_label_new (text);
+      gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+      gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
+      gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+      gtk_widget_show_all (GTK_WIDGET (about));
+    }
+  gtk_window_present (GTK_WINDOW (about));
 #endif
 }
 
