@@ -18,7 +18,8 @@
 
 #include <config.h>
 
-#include <gnome-db.h>
+#include <gtk/gtkmain.h>
+#include <libgnomedb/gnome-db-error-dialog.h>
 #include "../gb.h"
 
 /* Include the 21x21 icon pixmap for this widget, to be used in the palette */
@@ -32,7 +33,7 @@ static GbWidget gbwidget;
 
 static gchar *Title = "GnomeDbErrorDlg|GtkWindow::title";
 static gchar *Type = "GnomeDbErrorDlg|GtkWindow::type";
-static gchar *Position = "GnomeDbErrorDlg|GtkWindow::position";
+static gchar *Position = "GnomeDbErrorDlg|GtkWindow::window_position";
 static gchar *Modal = "GnomeDbErrorDlg|GtkWindow::modal";
 static gchar *DefaultWidth = "GnomeDbErrorDlg|GtkWindow::default_width";
 static gchar *DefaultHeight = "GnomeDbErrorDlg|GtkWindow::default_height";
@@ -42,6 +43,16 @@ static gchar *AutoShrink = "GnomeDbErrorDlg|GtkWindow::auto_shrink";
 static gchar *WMName = "GnomeDbErrorDlg|GtkWindow::wmclass_name";
 static gchar *WMClass = "GnomeDbErrorDlg|GtkWindow::wmclass_class";
 
+static gchar *Resizable = "GnomeDbErrorDlg|GtkWindow::resizable";
+static gchar *DestroyWithParent = "GnomeDbErrorDlg|GtkWindow::destroy_with_parent";
+static gchar *Icon = "GnomeDbErrorDlg|GtkWindow::icon";
+
+static gchar *Role = "GnomeDbErrorDlg|GtkWindow::role";
+static gchar *TypeHint = "GnomeDbErrorDlg|GtkWindow::type_hint";
+static gchar *SkipTaskbar = "GnomeDbErrorDlg|GtkWindow::skip_taskbar_hint";
+static gchar *SkipPager = "GnomeDbErrorDlg|GtkWindow::skip_pager_hint";
+static gchar *Decorated = "GnomeDbErrorDlg|GtkWindow::decorated";
+static gchar *Gravity = "GnomeDbErrorDlg|GtkWindow::gravity";
 
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
@@ -60,11 +71,7 @@ gb_gnome_db_errordlg_new (GbWidgetNewData *data)
   GtkWidget *widget;
   GList *elem;
 
-  widget = gnome_db_errordlg_new (NULL, _("Error"));
-
-  /* Stop the normal dismiss operation since it can cause a crash. */
-  elem = g_list_nth (GNOME_DIALOG (widget)->buttons, 2);
-  gtk_signal_disconnect_by_data (GTK_OBJECT (elem->data), widget);
+  widget = gnome_db_error_dialog_new (_("Error"));
 
   /* We connect a close signal handler which always returns TRUE so that
      the built-in close functionality is skipped. */
@@ -75,6 +82,9 @@ gb_gnome_db_errordlg_new (GbWidgetNewData *data)
   gtk_signal_connect (GTK_OBJECT (widget), "delete_event",
 		      GTK_SIGNAL_FUNC (editor_close_window), NULL);
 
+  gtk_object_set_data (GTK_OBJECT (widget), TypeHint,
+		       GINT_TO_POINTER (GLADE_TYPE_HINT_DIALOG_INDEX));
+
   return widget;
 }
 
@@ -90,7 +100,10 @@ gb_gnome_db_errordlg_create_properties (GtkWidget * widget, GbWidgetCreateArgDat
 					Title, Type, Position, Modal,
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, AutoShrink,
-					WMName, WMClass);
+					WMName, WMClass,
+					Resizable, DestroyWithParent, Icon,
+					Role, TypeHint, SkipTaskbar,
+					SkipPager, Decorated, Gravity);
 }
 
 
@@ -106,7 +119,10 @@ gb_gnome_db_errordlg_get_properties (GtkWidget *widget, GbWidgetGetArgData * dat
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 }
 
 
@@ -122,7 +138,10 @@ gb_gnome_db_errordlg_set_properties (GtkWidget * widget, GbWidgetSetArgData * da
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 }
 
 
@@ -132,13 +151,11 @@ gb_gnome_db_errordlg_set_properties (GtkWidget * widget, GbWidgetSetArgData * da
  * Add commands to aid in editing a GnomeDbErrorDlg, with signals pointing to
  * other functions in this file.
  */
-/*
 static void
 gb_gnome_db_errordlg_create_popup_menu (GtkWidget * widget, GbWidgetCreateMenuData * data)
 {
 
 }
-*/
 
 
 
@@ -153,12 +170,17 @@ gb_gnome_db_errordlg_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
   if (data->create_widget)
     {
       gchar *title;
+      gboolean translatable, context;
+      gchar *comments;
+
+      glade_util_get_translation_properties (widget, Title, &translatable,
+					     &comments, &context);
+      source_add_translator_comments (data, translatable, comments);
 
       title = GTK_WINDOW (widget)->title;
-      source_add (data, "  %s = gnome_db_errordlg_new (NULL, %s);\n",
+      source_add (data, "  %s = gnome_db_error_dialog_new (%s);\n",
 		  data->wname,
-		  title ? source_make_string (title, data->use_gettext)
-		        : "NULL");
+		  title ? source_make_string_full (title, data->use_gettext && translatable, context) : "NULL");
     }
 
   gb_widget_write_standard_source (widget, data);
@@ -168,7 +190,10 @@ gb_gnome_db_errordlg_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
 				   NULL, Type, Position, Modal,
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
-				   WMName, WMClass);
+				   WMName, WMClass,
+				   Resizable, DestroyWithParent, Icon,
+				   Role, TypeHint, SkipTaskbar,
+				   SkipPager, Decorated, Gravity);
 }
 
 
@@ -182,7 +207,8 @@ GbWidget*
 gb_gnome_db_errordlg_init ()
 {
   /* Initialise the GTK type */
-  gnome_db_errordlg_get_type();
+  volatile GtkType type;
+  type = gnome_db_error_dialog_get_type();
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct(&gbwidget);
@@ -197,9 +223,8 @@ gb_gnome_db_errordlg_init ()
   gbwidget.gb_widget_get_properties	= gb_gnome_db_errordlg_get_properties;
   gbwidget.gb_widget_set_properties	= gb_gnome_db_errordlg_set_properties;
   gbwidget.gb_widget_write_source	= gb_gnome_db_errordlg_write_source;
-/*
   gbwidget.gb_widget_create_popup_menu	= gb_gnome_db_errordlg_create_popup_menu;
-*/
+  gbwidget.gb_widget_destroy = gb_window_destroy;
 
   return &gbwidget;
 }
