@@ -47,6 +47,13 @@ static gchar *Resizable = "GnomeDbLoginDlg|GtkWindow::resizable";
 static gchar *DestroyWithParent = "GnomeDbLoginDlg|GtkWindow::destroy_with_parent";
 static gchar *Icon = "GnomeDbLoginDlg|GtkWindow::icon";
 
+static gchar *Role = "GnomeDbLoginDlg|GtkWindow::role";
+static gchar *TypeHint = "GnomeDbLoginDlg|GtkWindow::type_hint";
+static gchar *SkipTaskbar = "GnomeDbLoginDlg|GtkWindow::skip_taskbar_hint";
+static gchar *SkipPager = "GnomeDbLoginDlg|GtkWindow::skip_pager_hint";
+static gchar *Decorated = "GnomeDbLoginDlg|GtkWindow::decorated";
+static gchar *Gravity = "GnomeDbLoginDlg|GtkWindow::gravity";
+
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
  * to the function in the GbWidget struct at the end of this file.
@@ -77,6 +84,9 @@ gb_gnome_db_logindlg_new (GbWidgetNewData *data)
   gtk_signal_connect (GTK_OBJECT (widget), "delete_event",
 		      GTK_SIGNAL_FUNC (editor_close_window), NULL);
 
+  gtk_object_set_data (GTK_OBJECT (widget), TypeHint,
+		       GINT_TO_POINTER (GLADE_TYPE_HINT_DIALOG_INDEX));
+
   return widget;
 }
 
@@ -93,7 +103,9 @@ gb_gnome_db_logindlg_create_properties (GtkWidget * widget, GbWidgetCreateArgDat
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, AutoShrink,
 					WMName, WMClass,
-					Resizable, DestroyWithParent, Icon);
+					Resizable, DestroyWithParent, Icon,
+					Role, TypeHint, SkipTaskbar,
+					SkipPager, Decorated, Gravity);
 }
 
 
@@ -110,7 +122,9 @@ gb_gnome_db_logindlg_get_properties (GtkWidget *widget, GbWidgetGetArgData * dat
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
 				     WMName, WMClass,
-				     Resizable, DestroyWithParent, Icon);
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 }
 
 
@@ -127,7 +141,9 @@ gb_gnome_db_logindlg_set_properties (GtkWidget * widget, GbWidgetSetArgData * da
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
 				     WMName, WMClass,
-				     Resizable, DestroyWithParent, Icon);
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 }
 
 
@@ -155,12 +171,17 @@ gb_gnome_db_logindlg_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
   if (data->create_widget)
     {
       gchar *title;
+      gboolean translatable, context;
+      gchar *comments;
+
+      glade_util_get_translation_properties (widget, Title, &translatable,
+					     &comments, &context);
+      source_add_translator_comments (data, translatable, comments);
 
       title = GTK_WINDOW (widget)->title;
       source_add (data, "  %s = gnome_db_login_dialog_new (%s);\n",
 		  data->wname,
-		  title ? source_make_string (title, data->use_gettext)
-		        : "NULL");
+		  title ? source_make_string_full (title, data->use_gettext && translatable, context) : "NULL");
     }
 
   gb_widget_write_standard_source (widget, data);
@@ -171,7 +192,9 @@ gb_gnome_db_logindlg_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
 				   WMName, WMClass,
-				   Resizable, DestroyWithParent, Icon);
+				   Resizable, DestroyWithParent, Icon,
+				   Role, TypeHint, SkipTaskbar,
+				   SkipPager, Decorated, Gravity);
 }
 
 
