@@ -21,6 +21,7 @@
 #include <gnome.h>
 #include "../gb.h"
 #include "../tree.h"
+#include "../glade_gnome.h"
 
 /* Include the 21x21 icon pixmap for this widget, to be used in the palette */
 #include "../graphics/gnome-app.xpm"
@@ -32,13 +33,9 @@
 static GbWidget gbwidget;
 
 
-/* These are the special children of the widget. */
-static gchar *GnomeAppDock = "GnomeApp:dock";
-static gchar *GnomeAppAppBar = "GnomeApp:appbar";
-
 static gchar *Title = "GnomeApp|GtkWindow::title";
 static gchar *Type = "GnomeApp|GtkWindow::type";
-static gchar *Position = "GnomeApp|GtkWindow::position";
+static gchar *Position = "GnomeApp|GtkWindow::window_position";
 static gchar *Modal = "GnomeApp|GtkWindow::modal";
 static gchar *DefaultWidth = "GnomeApp|GtkWindow::default_width";
 static gchar *DefaultHeight = "GnomeApp|GtkWindow::default_height";
@@ -47,6 +44,18 @@ static gchar *Grow = "GnomeApp|GtkWindow::allow_grow";
 static gchar *AutoShrink = "GnomeApp|GtkWindow::auto_shrink";
 static gchar *WMName = "GnomeApp|GtkWindow::wmclass_name";
 static gchar *WMClass = "GnomeApp|GtkWindow::wmclass_class";
+
+static gchar *Resizable = "GnomeApp|GtkWindow::resizable";
+static gchar *DestroyWithParent = "GnomeApp|GtkWindow::destroy_with_parent";
+static gchar *Icon = "GnomeApp|GtkWindow::icon";
+
+static gchar *Role = "GnomeApp|GtkWindow::role";
+static gchar *TypeHint = "GnomeApp|GtkWindow::type_hint";
+static gchar *SkipTaskbar = "GnomeApp|GtkWindow::skip_taskbar_hint";
+static gchar *SkipPager = "GnomeApp|GtkWindow::skip_pager_hint";
+static gchar *Decorated = "GnomeApp|GtkWindow::decorated";
+static gchar *Gravity = "GnomeApp|GtkWindow::gravity";
+
 /* This is kept as object data as we don't want config info written when it is
    used in Glade. */
 static gchar *EnableLayoutConfig = "GnomeApp::enable_layout_config";
@@ -56,91 +65,7 @@ static gchar *EnableLayoutConfig = "GnomeApp::enable_layout_config";
 static gchar *StatusBar = "GnomeApp::status_bar";
 
 
-/* The indices come from GladeStockMenuItemValues in glade_gnome.c */
-static gint FileMenuIndices[] = { 3, 4, 5, 6, -1, 11 };
-static GnomeUIInfo FileMenu[] =
-{
-  GNOMEUIINFO_MENU_NEW_ITEM (N_("_New File"), N_("Create a new file"),
-			     NULL, NULL),
-  GNOMEUIINFO_MENU_OPEN_ITEM (NULL, NULL),
-  GNOMEUIINFO_MENU_SAVE_ITEM (NULL, NULL),
-  GNOMEUIINFO_MENU_SAVE_AS_ITEM (NULL, NULL),
-  GNOMEUIINFO_SEPARATOR,
-  GNOMEUIINFO_MENU_EXIT_ITEM (NULL, NULL),
-  GNOMEUIINFO_END
-};
-
-static gint EditMenuIndices[] = { 14, 15, 16, 18, -1, 24 };
-static GnomeUIInfo EditMenu[] =
-{
-  GNOMEUIINFO_MENU_CUT_ITEM (NULL, NULL),
-  GNOMEUIINFO_MENU_COPY_ITEM (NULL, NULL),
-  GNOMEUIINFO_MENU_PASTE_ITEM (NULL, NULL),
-  GNOMEUIINFO_MENU_CLEAR_ITEM (NULL, NULL),
-  GNOMEUIINFO_SEPARATOR,
-  GNOMEUIINFO_MENU_PROPERTIES_ITEM (NULL, NULL),
-  GNOMEUIINFO_END
-};
-
-static gint ViewMenuIndices[] = { -1 };
-static GnomeUIInfo ViewMenu[] =
-{
-  GNOMEUIINFO_END
-};
-
-static gint SettingsMenuIndices[] = { 29 };
-static GnomeUIInfo SettingsMenu[] =
-{
-  GNOMEUIINFO_MENU_PREFERENCES_ITEM (NULL, NULL),
-  GNOMEUIINFO_END
-};
-
-static gint HelpMenuIndices[] = { 38 };
-static GnomeUIInfo HelpMenu[] =
-{
-  GNOMEUIINFO_MENU_ABOUT_ITEM (NULL, NULL),
-  GNOMEUIINFO_END
-};
-
-static gint MainMenuIndices[] = { 2, 13, 26, 28, 37 };
-static GnomeUIInfo MainMenu[] =
-{
-  GNOMEUIINFO_MENU_FILE_TREE (FileMenu),
-  GNOMEUIINFO_MENU_EDIT_TREE (EditMenu),
-  GNOMEUIINFO_MENU_VIEW_TREE (ViewMenu),
-  GNOMEUIINFO_MENU_SETTINGS_TREE (SettingsMenu),
-  GNOMEUIINFO_MENU_HELP_TREE (HelpMenu),
-  GNOMEUIINFO_END
-};
-
-
-/* These are the indices of the icons in the toolbar items below, in the
-   GladeStockPixmapChoices and related arrays. */
-static gint ToolBarIndices[] = { 1, 2, 5 };
-
-static GnomeUIInfo ToolBar[] =
-{
-  {
-    GNOME_APP_UI_ITEM, N_("New"), N_("New File"), 
-    NULL, NULL, NULL,
-    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_NEW, 0, 0, NULL
-  },
-  {
-    GNOME_APP_UI_ITEM, N_("Open"), N_("Open File"), 
-    NULL, NULL, NULL,
-    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_OPEN, 0, 0, NULL
-  },
-  {
-    GNOME_APP_UI_ITEM, N_("Save"), N_("Save File"), 
-    NULL, NULL, NULL,
-    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_SAVE, 0, 0, NULL
-  },
-  GNOMEUIINFO_END
-};
-
-
 static void gb_gnome_app_setup_initial_app (GtkWidget *widget);
-static void gb_gnome_app_setup_menu_items (GnomeUIInfo *uiinfo, gint *indices);
 
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
@@ -166,19 +91,19 @@ gb_gnome_app_new (GbWidgetNewData *data)
 		       GINT_TO_POINTER (TRUE));
 
   /* Turn off the automatic loading/saving of the configuration, and
-     destroy the GnomeDockLayout, so that when items are added they are
+     destroy the BonoboDockLayout, so that when items are added they are
      added straight away. If we don't do this, items don't get added to
      the widget tree. */
   gnome_app_enable_layout_config (GNOME_APP (new_widget), FALSE);
-  gtk_object_unref (GTK_OBJECT (GNOME_APP (new_widget)->layout));
+  g_object_unref (G_OBJECT (GNOME_APP (new_widget)->layout));
   GNOME_APP (new_widget)->layout = NULL;
 
   gtk_signal_connect (GTK_OBJECT (new_widget), "delete_event",
 		      GTK_SIGNAL_FUNC (editor_close_window), NULL);
 
   gb_widget_create_from (GNOME_APP (new_widget)->dock,
-			 data->action == GB_CREATING ? "GnomeDock" : NULL);
-  gb_widget_set_child_name (GNOME_APP (new_widget)->dock, GnomeAppDock);
+			 data->action == GB_CREATING ? "BonoboDock" : NULL);
+  gb_widget_set_child_name (GNOME_APP (new_widget)->dock, GladeChildGnomeAppDock);
 
   if (data->action == GB_CREATING)
     gb_gnome_app_setup_initial_app (new_widget);
@@ -187,16 +112,39 @@ gb_gnome_app_new (GbWidgetNewData *data)
 }
 
 
+static void
+gb_gnome_app_add_toolbar_button (GtkToolbar *toolbar,
+				 const gchar *stock_id,
+				 const gchar *tooltip)
+{
+  GtkWidget *button;
+  GladeWidgetData *wdata;
+
+  button = gb_widget_new ("GtkToolButton", NULL);
+
+  gtk_tool_button_set_stock_id (GTK_TOOL_BUTTON (button), stock_id);
+  gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (button), NULL);
+  gtk_tool_button_set_label (GTK_TOOL_BUTTON (button), NULL);
+
+  gtk_object_set_data_full (GTK_OBJECT (button),
+			    GladeToolButtonStockIDKey,
+			    g_strdup (stock_id), g_free);
+
+  wdata = gtk_object_get_data (GTK_OBJECT (button), GB_WIDGET_DATA_KEY);
+  wdata->tooltip = g_strdup (tooltip);
+
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (button), -1);
+}
+
+
 static void
 gb_gnome_app_setup_initial_app (GtkWidget *widget)
 {
   GtkWidget *placeholder, *toolbar;
-  GnomeUIInfo *uiinfo;
-  gint i;
 
   /* We create a standard menubar and toolbar which the user can edit or
      simply delete anything they don't want. */
-  gnome_app_create_menus (GNOME_APP (widget), MainMenu);
+  glade_gnome_setup_initial_menus (widget);
 
   /* We need to get rid of the accelerators, since they aren't used, and
      they would override our default accelerators. */
@@ -205,146 +153,36 @@ gb_gnome_app_setup_initial_app (GtkWidget *widget)
 
   /* FIXME: GnomeLibs bug workaround. It sets the initial border width
      to -2, which causes us lots of problems. */
+#if 0
   gtk_container_set_border_width (GTK_CONTAINER (GNOME_APP (widget)->menubar),
 				  0);
+#endif
   gb_widget_create_from (GNOME_APP (widget)->menubar, "GtkMenubar");
-  gb_widget_create_from (GNOME_APP (widget)->menubar->parent, "GnomeDockItem");
-
-  /* Turn all the menus & menuitems into GbWidgets and add default
-     handlers. */
-  gb_gnome_app_setup_menu_items (FileMenu, FileMenuIndices);
-  gb_gnome_app_setup_menu_items (EditMenu, EditMenuIndices);
-  gb_gnome_app_setup_menu_items (ViewMenu, ViewMenuIndices);
-  gb_gnome_app_setup_menu_items (SettingsMenu, SettingsMenuIndices);
-  gb_gnome_app_setup_menu_items (HelpMenu, HelpMenuIndices);
-  gb_gnome_app_setup_menu_items (MainMenu, MainMenuIndices);
-
+  gb_widget_create_from (GNOME_APP (widget)->menubar->parent, "BonoboDockItem");
 
   /* Toolbar */
-  toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
-  gnome_app_fill_toolbar (GTK_TOOLBAR (toolbar), ToolBar,
-			  GNOME_APP (widget)->accel_group);
+  toolbar = gtk_toolbar_new ();
   gnome_app_set_toolbar (GNOME_APP (widget), GTK_TOOLBAR (toolbar));
   gb_widget_create_from (toolbar, "GtkToolbar");
-  gb_widget_create_from (toolbar->parent, "GnomeDockItem");
+  gb_widget_create_from (toolbar->parent, "BonoboDockItem");
 
-  for (uiinfo = ToolBar, i = 0;
-       uiinfo->type != GNOME_APP_UI_ENDOFINFO;
-       uiinfo++, i++)
-    {
-      if (uiinfo->widget)
-	{
-	  gb_widget_create_from (uiinfo->widget, "GtkButton");
-	  gb_widget_set_child_name (uiinfo->widget, "Toolbar:button");
-	  gtk_object_set_data (GTK_OBJECT (uiinfo->widget), "GbStockIcon",
-			       GINT_TO_POINTER (ToolBarIndices[i]));
+  gb_gnome_app_add_toolbar_button (GTK_TOOLBAR (toolbar), GTK_STOCK_NEW,
+				   _("New File"));
+  gb_gnome_app_add_toolbar_button (GTK_TOOLBAR (toolbar), GTK_STOCK_OPEN,
+				   _("Open File"));
+  gb_gnome_app_add_toolbar_button (GTK_TOOLBAR (toolbar), GTK_STOCK_SAVE,
+				   _("Save File"));
 
-	  /* Copy the tooltip to our GladeWidgetData struct. */
-	  if (uiinfo->hint)
-	    {
-	      GladeWidgetData *wdata;
-
-	      wdata = gtk_object_get_data (GTK_OBJECT (uiinfo->widget),
-					   GB_WIDGET_DATA_KEY);
-	      wdata->tooltip = g_strdup (_(uiinfo->hint));
-	    }
-	}
-    }
 
   /* Statusbar */
   gnome_app_set_statusbar (GNOME_APP (widget),
 			   gb_widget_new ("GnomeAppBar", widget));
-  gb_widget_set_child_name (GNOME_APP (widget)->statusbar, GnomeAppAppBar);
+  gb_widget_set_child_name (GNOME_APP (widget)->statusbar, GladeChildGnomeAppBar);
 
   /* We need to size the placeholders or the dialog is very small. */
   placeholder = editor_new_placeholder ();
   gtk_widget_set_usize (placeholder, 300, 200);
   gnome_app_set_contents (GNOME_APP (widget), placeholder);
-  gb_widget_set_child_name (placeholder, "GnomeDock:contents");
-}
-
-
-static void
-gb_gnome_app_setup_menu_items (GnomeUIInfo *uiinfo, gint *indices)
-{
-  gint i = 0;
-  gchar *name, *src, *dest;
-
-  while (uiinfo->type != GNOME_APP_UI_ENDOFINFO)
-    {
-      if (uiinfo->widget)
-	{
-	  if (indices[i] == -1)
-	    {
-	      gb_widget_create_from (uiinfo->widget, "separator");
-	    }
-	  else
-	    {
-	      name = g_malloc (strlen (uiinfo->label));
-	      /* Convert spaces to underscores, and ignore periods (e.g. in
-		 "Open...") and underscores (e.g. in "_Open"). */
-	      for (src = uiinfo->label, dest = name; *src; src++)
-		{
-		  if (*src == ' ')
-		    *dest++ = '_';
-		  else if (*src == '.')
-		    continue;
-		  else if (*src == '_')
-		    continue;
-		  else
-		    *dest++ = *src;
-		}
-	      *dest = '\0';
-
-	      gb_widget_create_from (uiinfo->widget, name);
-
-	      gtk_object_set_data (GTK_OBJECT (uiinfo->widget),
-				   GladeMenuItemStockIndexKey,
-				   GINT_TO_POINTER (indices[i]));
-
-	      /* If the item has a child menu, turn it into a GbWidget.
-		 If not, add a default handler to the item. */
-	      if (GTK_IS_MENU_ITEM (uiinfo->widget)
-		  && GTK_MENU_ITEM (uiinfo->widget)->submenu)
-		{
-		  gchar *submenu_name;
-
-		  submenu_name = g_strdup_printf ("%s_menu",
-						  uiinfo->widget->name);
-		  gtk_widget_set_name (GTK_MENU_ITEM (uiinfo->widget)->submenu,
-				       submenu_name);
-		  g_free (submenu_name);
-		  gb_widget_create_from (GTK_MENU_ITEM (uiinfo->widget)->submenu, NULL);
-		}
-	      else
-		{
-		  GladeWidgetData *wdata;
-		  GladeSignal *signal;
-		  gchar *start = "on_", *end = "_activate";
-
-		  signal = g_new (GladeSignal, 1);
-		  signal->name = g_strdup ("activate");
-		  /* Generate a default handler name. */
-		  signal->handler = g_malloc (strlen (uiinfo->widget->name) + 1
-					      + strlen (start) + strlen (end));
-		  strcpy (signal->handler, start);
-		  strcat (signal->handler, uiinfo->widget->name);
-		  strcat (signal->handler, end);
-
-		  signal->object = NULL;
-		  signal->after = FALSE;
-		  signal->data = NULL;
-		  signal->last_modification_time = time (NULL);
-
-		  wdata = gtk_object_get_data (GTK_OBJECT (uiinfo->widget),
-					       GB_WIDGET_DATA_KEY);
-		  wdata->signals = g_list_append (wdata->signals, signal);
-		}
-	    }
-	}
-      uiinfo++;
-      i++;
-    }
 }
 
 
@@ -358,7 +196,10 @@ gb_gnome_app_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data
 					Title, Type, Position, Modal,
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, AutoShrink,
-					WMName, WMClass);
+					WMName, WMClass,
+					Resizable, DestroyWithParent, Icon,
+					Role, TypeHint, SkipTaskbar,
+					SkipPager, Decorated, Gravity);
   property_add_bool (StatusBar, _("Status Bar:"),
 		     _("If the window has a status bar"));
   property_add_bool (EnableLayoutConfig, _("Store Config:"),
@@ -378,7 +219,10 @@ gb_gnome_app_get_properties (GtkWidget *widget, GbWidgetGetArgData * data)
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 
   if (data->action == GB_SHOWING)
     {
@@ -405,7 +249,10 @@ gb_gnome_app_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 
   if (data->action == GB_APPLYING)
     {
@@ -420,7 +267,7 @@ gb_gnome_app_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 					   gb_widget_new ("GnomeAppBar",
 							  widget));
 		  gb_widget_set_child_name (GNOME_APP (widget)->statusbar,
-					    GnomeAppAppBar);
+					    GladeChildGnomeAppBar);
 		  tree_add_widget (GNOME_APP (widget)->statusbar);
 		}
 	    }
@@ -474,6 +321,8 @@ static void
 gb_gnome_app_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 {
   gchar *appname;
+  gboolean translatable, context;
+  gchar *comments;
 
   if (data->create_widget)
     {
@@ -481,14 +330,19 @@ gb_gnome_app_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
       appname = appname ? source_make_string (appname, FALSE) : "\"\"";
       appname = g_strdup (appname);
 
+      glade_util_get_translation_properties (widget, Title, &translatable,
+					     &comments, &context);
+      source_add_translator_comments (data, translatable, comments);
+
       /* Note that this assumes that we use the same appname for each GnomeApp.
 	 FIXME: I think this is correct, but I'm not sure.
 	 Note also that we don't translate the project name. Maybe we should.*/
       source_add (data, "  %s = gnome_app_new (%s, %s);\n", data->wname,
 		  appname,
 		  GTK_WINDOW (widget)->title
-		  ? source_make_string (GTK_WINDOW (widget)->title,
-					data->use_gettext)
+		  ? source_make_string_full (GTK_WINDOW (widget)->title,
+					     data->use_gettext && translatable,
+					     context)
 		  : "NULL");
       g_free (appname);
     }
@@ -500,7 +354,10 @@ gb_gnome_app_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 				   NULL, Type, Position, Modal,
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
-				   WMName, WMClass);
+				   WMName, WMClass,
+				   Resizable, DestroyWithParent, Icon,
+				   Role, TypeHint, SkipTaskbar,
+				   SkipPager, Decorated, Gravity);
 
   if (!GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
 					     EnableLayoutConfig)))
@@ -523,7 +380,7 @@ gb_gnome_app_write_add_child_source (GtkWidget * parent,
 
   child_name = gb_widget_get_child_name (child);
 
-  if (child_name && (!strcmp (child_name, GnomeAppAppBar)))
+  if (child_name && (!strcmp (child_name, GladeChildGnomeAppBar)))
     {
       source_add (data,
 		  "  gnome_app_set_statusbar (GNOME_APP (%s), %s);\n",
@@ -542,9 +399,12 @@ static void
 gb_gnome_app_add_child (GtkWidget *widget, GtkWidget * child,
 			GbWidgetSetArgData *data)
 {
-  gchar *child_name = load_get_value (data, "child_name");
+  gchar *child_name = NULL;
 
-  if (child_name && (!strcmp (child_name, GnomeAppAppBar)))
+  if (data->child_info)
+    child_name = data->child_info->internal_child;
+
+  if (child_name && (!strcmp (child_name, GladeChildGnomeAppBar)))
     {
       gnome_app_set_statusbar (GNOME_APP (widget), child);
     }
@@ -555,7 +415,7 @@ static GtkWidget *
 gb_gnome_app_get_child (GtkWidget * widget,
 			const gchar * child_name)
 {
-  if (!strcmp (child_name, GnomeAppDock))
+  if (!strcmp (child_name, GladeChildGnomeAppDock))
     return GNOME_APP (widget)->dock;
   else
     return NULL;
@@ -571,7 +431,8 @@ GbWidget*
 gb_gnome_app_init ()
 {
   /* Initialise the GTK type */
-  gnome_app_get_type();
+  volatile GtkType type;
+  type = gnome_app_get_type();
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct(&gbwidget);
@@ -589,6 +450,7 @@ gb_gnome_app_init ()
   gbwidget.gb_widget_set_properties	= gb_gnome_app_set_properties;
   gbwidget.gb_widget_write_source	= gb_gnome_app_write_source;
   gbwidget.gb_widget_write_add_child_source = gb_gnome_app_write_add_child_source;
+  gbwidget.gb_widget_destroy = gb_window_destroy;
 /*
   gbwidget.gb_widget_create_popup_menu	= gb_gnome_app_create_popup_menu;
 */
