@@ -47,6 +47,13 @@ static gchar *Resizable = "GnomeDialog|GtkWindow::resizable";
 static gchar *DestroyWithParent = "GnomeDialog|GtkWindow::destroy_with_parent";
 static gchar *Icon = "GnomeDialog|GtkWindow::icon";
 
+static gchar *Role = "GnomeDialog|GtkWindow::role";
+static gchar *TypeHint = "GnomeDialog|GtkWindow::type_hint";
+static gchar *SkipTaskbar = "GnomeDialog|GtkWindow::skip_taskbar_hint";
+static gchar *SkipPager = "GnomeDialog|GtkWindow::skip_pager_hint";
+static gchar *Decorated = "GnomeDialog|GtkWindow::decorated";
+static gchar *Gravity = "GnomeDialog|GtkWindow::gravity";
+
 static gchar *AutoClose = "GnomeDialog::auto_close";
 static gchar *HideOnClose = "GnomeDialog::hide_on_close";
 
@@ -121,6 +128,9 @@ gb_gnome_dialog_new (GbWidgetNewData *data)
   gtk_signal_connect (GTK_OBJECT (new_widget), "delete_event",
 		      GTK_SIGNAL_FUNC (editor_close_window), NULL);
 
+  gtk_object_set_data (GTK_OBJECT (new_widget), TypeHint,
+		       GINT_TO_POINTER (GLADE_TYPE_HINT_DIALOG_INDEX));
+
   return new_widget;
 }
 
@@ -136,7 +146,9 @@ gb_gnome_dialog_create_properties (GtkWidget * widget, GbWidgetCreateArgData * d
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, AutoShrink,
 					WMName, WMClass,
-					Resizable, DestroyWithParent, Icon);
+					Resizable, DestroyWithParent, Icon,
+					Role, TypeHint, SkipTaskbar,
+					SkipPager, Decorated, Gravity);
   property_add_bool (AutoClose, _("Auto Close:"),
 		     _("If the dialog closes when any button is clicked"));
   property_add_bool (HideOnClose, _("Hide on Close:"),
@@ -157,7 +169,9 @@ gb_gnome_dialog_get_properties (GtkWidget *widget, GbWidgetGetArgData * data)
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
 				     WMName, WMClass,
-				     Resizable, DestroyWithParent, Icon);
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 
   gb_widget_output_bool (data, AutoClose, GNOME_DIALOG (widget)->click_closes);
   gb_widget_output_bool (data, HideOnClose, GNOME_DIALOG (widget)->just_hide);
@@ -179,7 +193,9 @@ gb_gnome_dialog_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
 				     WMName, WMClass,
-				     Resizable, DestroyWithParent, Icon);
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 
   auto_close = gb_widget_input_bool (data, AutoClose);
   if (data->apply)
@@ -216,15 +232,21 @@ static void
 gb_gnome_dialog_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 {
   gchar *wname, *child_name;
+  gboolean translatable, context;
+  gchar *comments;
 
   if (data->create_widget)
     {
       gchar *title;
 
+      glade_util_get_translation_properties (widget, Title, &translatable,
+					     &comments, &context);
+      source_add_translator_comments (data, translatable, comments);
+
       title = GTK_WINDOW (widget)->title;
       source_add (data, "  %s = gnome_dialog_new (%s, NULL);\n",
 		  data->wname,
-		  title ? source_make_string (title, data->use_gettext)
+		  title ? source_make_string_full (title, data->use_gettext && translatable, context)
 		        : "NULL");
     }
 
@@ -236,7 +258,9 @@ gb_gnome_dialog_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
 				   WMName, WMClass,
-				   Resizable, DestroyWithParent, Icon);
+				   Resizable, DestroyWithParent, Icon,
+				   Role, TypeHint, SkipTaskbar,
+				   SkipPager, Decorated, Gravity);
 
   if (GNOME_DIALOG (widget)->click_closes)
     {
