@@ -31,12 +31,8 @@
  */
 static GbWidget gbwidget;
 
-/* These are the special children of the widget. */
-static gchar *DialogVBox = "GnomeDialog:vbox";
-static gchar *DialogActionArea = "GnomeDialog:action_area";
-
 static gchar *Title = "GnomeMessageBox|GtkWindow::title";
-static gchar *Position = "GnomeMessageBox|GtkWindow::position";
+static gchar *Position = "GnomeMessageBox|GtkWindow::window_position";
 static gchar *Modal = "GnomeMessageBox|GtkWindow::modal";
 static gchar *DefaultWidth = "GnomeMessageBox|GtkWindow::default_width";
 static gchar *DefaultHeight = "GnomeMessageBox|GtkWindow::default_height";
@@ -46,6 +42,17 @@ static gchar *AutoShrink = "GnomeMessageBox|GtkWindow::auto_shrink";
 static gchar *WMName = "GnomeMessageBox|GtkWindow::wmclass_name";
 static gchar *WMClass = "GnomeMessageBox|GtkWindow::wmclass_class";
 
+static gchar *Resizable = "GnomeMessageBox|GtkWindow::resizable";
+static gchar *DestroyWithParent = "GnomeMessageBox|GtkWindow::destroy_with_parent";
+static gchar *Icon = "GnomeMessageBox|GtkWindow::icon";
+
+static gchar *Role = "GnomeMessageBox|GtkWindow::role";
+static gchar *TypeHint = "GnomeMessageBox|GtkWindow::type_hint";
+static gchar *SkipTaskbar = "GnomeMessageBox|GtkWindow::skip_taskbar_hint";
+static gchar *SkipPager = "GnomeMessageBox|GtkWindow::skip_pager_hint";
+static gchar *Decorated = "GnomeMessageBox|GtkWindow::decorated";
+static gchar *Gravity = "GnomeMessageBox|GtkWindow::gravity";
+
 static gchar *MessageBoxType = "GnomeMessageBox::message_box_type";
 static gchar *Message = "GnomeMessageBox::message";
 
@@ -70,19 +77,16 @@ static const gchar *GbMessageBoxTypeSymbols[] =
   "GNOME_MESSAGE_BOX_QUESTION",
   "GNOME_MESSAGE_BOX_GENERIC"
 };
-static const gchar* GbMessageBoxTypePixmapFilenames[] =
+static const gchar* GbMessageBoxTypePixmapStockIDs[] =
 {
-  "gnome-info.png",
-  "gnome-warning.png",
-  "gnome-error.png",
-  "gnome-question.png",
-  "gnome-default-dlg.png",
+  GTK_STOCK_DIALOG_INFO,
+  GTK_STOCK_DIALOG_WARNING,
+  GTK_STOCK_DIALOG_ERROR,
+  GTK_STOCK_DIALOG_QUESTION,
+  ""
 };
 
 
-/* Copied from gnome-dialog.c to workaround initialization problems. */
-static void gnome_dialog_init_action_area (GnomeDialog * dialog);
-
 static void get_message_box_widgets (GtkWidget *dialog,
 				     GtkWidget **pixmap,
 				     GtkWidget **label);
@@ -107,38 +111,43 @@ gb_gnome_message_box_new (GbWidgetNewData *data)
 {
   GtkWidget *new_widget;
   GList *elem;
+  GtkWidget *aa;
 
   if (data->action == GB_CREATING)
     {
       /* When creating a new dialog, we add a few standard buttons, which
 	 the user can change/delete easily. */
       new_widget = gnome_message_box_new ("", GNOME_MESSAGE_BOX_INFO,
-					  GNOME_STOCK_BUTTON_OK, NULL);
+					  GTK_STOCK_OK, NULL);
 
       /* Now turn the buttons into GbWidgets so the user can edit them. */
       elem = GNOME_DIALOG (new_widget)->buttons;
       gb_widget_create_from (GTK_WIDGET (elem->data), "button");
-      gtk_object_set_data (GTK_OBJECT (elem->data), "GtkButton::stock_button",
-			   GINT_TO_POINTER (GladeStockButtonOK));
+      gtk_object_set_data (GTK_OBJECT (elem->data), GladeButtonStockIDKey,
+			   GTK_STOCK_OK);
     }
   else
     {
-      new_widget = gnome_message_box_new ("",  GNOME_MESSAGE_BOX_INFO, NULL);
-
-      /* FIXME: Kludge to initialize the action area so we can refer to it
-	 below. */
-      gnome_dialog_init_action_area (GNOME_DIALOG (new_widget));
+      /* FIXME: We create it with an OK button, and then remove the button,
+	 to work around a bug in gnome_message_box_new() - it tries to set the
+	 keyboard focus to the last button, which may not exist. It also
+	 ensures that gnome_dialog_init_action_area() has been called. */
+      new_widget = gnome_message_box_new ("",  GNOME_MESSAGE_BOX_INFO,
+					  GNOME_STOCK_BUTTON_OK, NULL);
+      gtk_container_remove (GTK_CONTAINER (GNOME_DIALOG (new_widget)->action_area),
+			    GNOME_DIALOG (new_widget)->buttons->data);
+      GNOME_DIALOG (new_widget)->buttons = NULL;
     }
 
+  aa = GNOME_DIALOG (new_widget)->action_area;
+
   gb_widget_create_from (GNOME_DIALOG (new_widget)->vbox,
 			 data->action == GB_CREATING ? "dialog-vbox" : NULL);
-  gb_widget_set_child_name (GNOME_DIALOG (new_widget)->vbox, DialogVBox);
-
-  gb_widget_create_from (GNOME_DIALOG (new_widget)->action_area,
+  gb_widget_set_child_name (GNOME_DIALOG (new_widget)->vbox, GladeChildDialogVBox);
+  gb_widget_create_from (aa,
 			 data->action == GB_CREATING ? "dialog-action_area"
-						     : NULL);
-  gb_widget_set_child_name (GNOME_DIALOG (new_widget)->action_area,
-			    DialogActionArea);
+			 : NULL);
+  gb_widget_set_child_name (aa, GladeChildDialogActionArea);
 
   /* We connect a close signal handler which always returns TRUE so that
      the built-in close functionality is skipped. */
@@ -149,39 +158,13 @@ gb_gnome_message_box_new (GbWidgetNewData *data)
   gtk_signal_connect (GTK_OBJECT (new_widget), "delete_event",
 		      GTK_SIGNAL_FUNC (editor_close_window), NULL);
 
+  gtk_object_set_data (GTK_OBJECT (new_widget), TypeHint,
+		       GINT_TO_POINTER (GLADE_TYPE_HINT_DIALOG_INDEX));
+
   return new_widget;
 }
 
 
-/* FIXME: A kludge to initialize the action area, needed because
-   gnome_message_box_new() doesn't do this. Copied from gnome-dialog.c */
-static void
-gnome_dialog_init_action_area (GnomeDialog * dialog)
-{
-  GtkWidget * separator;
-
-  if (dialog->action_area)
-    return;
-
-  dialog->action_area = gtk_hbutton_box_new ();
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog->action_area),
-			     gnome_preferences_get_button_layout());
-
-  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog->action_area), 
-			      GNOME_PAD);
-
-  gtk_box_pack_end (GTK_BOX (dialog->vbox), dialog->action_area, 
-		    FALSE, TRUE, 0);
-  gtk_widget_show (dialog->action_area);
-
-  separator = gtk_hseparator_new ();
-  gtk_box_pack_end (GTK_BOX (dialog->vbox), separator, 
-		      FALSE, TRUE,
-		      GNOME_PAD_SMALL);
-  gtk_widget_show (separator);
-}
-
-
 /*
  * Creates the components needed to edit the extra properties of this widget.
  */
@@ -191,14 +174,17 @@ gb_gnome_message_box_create_properties (GtkWidget * widget, GbWidgetCreateArgDat
   property_add_choice (MessageBoxType, _("Message Type:"),
 		       _("The type of the message box"),
 		       GbMessageBoxTypeChoices);
-  property_add_text (Message, _("Message:"), _("The message to display"), 4);
+  property_add_text (Message, _("Message:"), _("The message to display"), 5);
 
   /* We don't allow setting of the title, so we pass NULL here. */
   gb_window_create_standard_properties (widget, data,
 					Title, NULL, Position, Modal,
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, AutoShrink,
-					WMName, WMClass);
+					WMName, WMClass,
+					Resizable, DestroyWithParent, Icon,
+					Role, TypeHint, SkipTaskbar,
+					SkipPager, Decorated, Gravity);
 
   property_add_bool (AutoClose, _("Auto Close:"),
 		     _("If the dialog closes when any button is clicked"));
@@ -216,7 +202,7 @@ static void
 gb_gnome_message_box_get_properties (GtkWidget *widget, GbWidgetGetArgData * data)
 {
   GtkWidget *pixmap, *label;
-  gchar *label_text;
+  const gchar *label_text;
   gint type_index;
 
   get_message_box_widgets (widget, &pixmap, &label);
@@ -228,15 +214,17 @@ gb_gnome_message_box_get_properties (GtkWidget *widget, GbWidgetGetArgData * dat
   gb_widget_output_choice (data, MessageBoxType, type_index,
 			   GbMessageBoxTypeSymbols[type_index]);
 
-  gtk_label_get (GTK_LABEL (label), &label_text);
+  label_text = gtk_label_get_text (GTK_LABEL (label));
   gb_widget_output_translatable_text (data, Message, label_text);
 
   gb_window_get_standard_properties (widget, data,
 				     Title, NULL, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
-
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
   gb_widget_output_bool (data, AutoClose, GNOME_DIALOG (widget)->click_closes);
   gb_widget_output_bool (data, HideOnClose, GNOME_DIALOG (widget)->just_hide);
 }
@@ -259,10 +247,16 @@ get_message_box_widgets (GtkWidget *dialog,
   if (!vbox || !GTK_IS_VBOX (vbox))
     return;
 
-  elem = GTK_BOX (vbox)->children;
-  child = (GtkBoxChild*)elem->data;
-  if (!child || !GTK_IS_HBOX (child->widget))
+  for (elem = GTK_BOX (vbox)->children; elem; elem = elem->next)
+    {
+      child = (GtkBoxChild*)elem->data;
+      if (GTK_IS_HBOX (child->widget))
+	break;
+    }
+
+  if (!elem)
     return;
+
   hbox = child->widget;
 
   elem = GTK_BOX (hbox)->children;
@@ -271,7 +265,7 @@ get_message_box_widgets (GtkWidget *dialog,
       child = (GtkBoxChild*)elem->data;
       if (GTK_IS_LABEL (child->widget))
 	*label = child->widget;
-      if (GNOME_IS_PIXMAP (child->widget))
+      if (GTK_IS_IMAGE (child->widget))
 	*pixmap = child->widget;
 
       elem = elem->next;
@@ -312,7 +306,10 @@ gb_gnome_message_box_set_properties (GtkWidget * widget, GbWidgetSetArgData * da
 				     Title, NULL, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 
   auto_close = gb_widget_input_bool (data, AutoClose);
   if (data->apply)
@@ -331,7 +328,6 @@ set_message_box_type (GtkWidget *dialog,
 		      gchar *type_name)
 {
   gint i, type_index = 0; /* Set to 0 in case we can't find it. */
-  gchar *filename;
 
   /* Find out the index of the type. */
   for (i = 0; GbMessageBoxTypeChoices[i]; i++)
@@ -347,9 +343,8 @@ set_message_box_type (GtkWidget *dialog,
   gtk_object_set_data (GTK_OBJECT (dialog), MessageBoxType,
 		       GINT_TO_POINTER (type_index));
 
-  filename = gnome_unconditional_pixmap_file (GbMessageBoxTypePixmapFilenames[type_index]);
-  gnome_pixmap_load_file (GNOME_PIXMAP (pixmap), filename);
-  g_free (filename);
+  g_object_set (G_OBJECT (pixmap), "stock",
+		GbMessageBoxTypePixmapStockIDs[type_index], NULL);
 }
 
 
@@ -377,9 +372,11 @@ static void
 gb_gnome_message_box_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 {
   GtkWidget *pixmap, *label;
-  gchar *label_text;
+  const gchar *label_text;
   gint type_index;
   gchar *wname, *child_name;
+  gboolean translatable, context;
+  gchar *comments;
 
   get_message_box_widgets (widget, &pixmap, &label);
   g_return_if_fail (pixmap != NULL);
@@ -387,16 +384,27 @@ gb_gnome_message_box_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
 
   type_index = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
 						     MessageBoxType));
-  gtk_label_get (GTK_LABEL (label), &label_text);
+  label_text = gtk_label_get_text (GTK_LABEL (label));
 
   if (data->create_widget)
     {
+      glade_util_get_translation_properties (widget, Message, &translatable,
+					     &comments, &context);
+      source_add_translator_comments (data, translatable, comments);
+
       source_add (data,
 		  "  %s = gnome_message_box_new (%s,\n"
 		  "                              %s, NULL);\n",
 		  data->wname,
-		  source_make_string (label_text, data->use_gettext),
+		  source_make_string_full (label_text, data->use_gettext && translatable, context),
 		  GbMessageBoxTypeSymbols[type_index]);
+
+#if 0
+      source_add (data,
+		  "  gtk_container_remove (GTK_CONTAINER (GNOME_DIALOG (%s)->action_area), GNOME_DIALOG (%s)->buttons->data);\n"
+		  "  GNOME_DIALOG (%s)->buttons = NULL;\n",
+		  data->wname, data->wname, data->wname);
+#endif
     }
 
   gb_widget_write_standard_source (widget, data);
@@ -405,7 +413,10 @@ gb_gnome_message_box_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
 				   Title, NULL, Position, Modal,
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
-				   WMName, WMClass);
+				   WMName, WMClass,
+				   Resizable, DestroyWithParent, Icon,
+				   Role, TypeHint, SkipTaskbar,
+				   SkipPager, Decorated, Gravity);
 
   if (!GNOME_DIALOG (widget)->click_closes)
     {
@@ -421,7 +432,6 @@ gb_gnome_message_box_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
 		  data->wname);
     }
 
-
   /* We output the source code for the children here, since the code should
      not include calls to create the widgets. We need to specify that the
      names used are like: "GTK_DIALOG (<dialog-name>)->vbox".
@@ -430,7 +440,7 @@ gb_gnome_message_box_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
   wname = g_strdup (data->wname);
 
   source_add (data, "\n");
-  child_name = gtk_widget_get_name (GNOME_DIALOG (widget)->vbox);
+  child_name = (char*) gtk_widget_get_name (GNOME_DIALOG (widget)->vbox);
   child_name = source_create_valid_identifier (child_name);
   source_add (data, "  %s = GNOME_DIALOG (%s)->vbox;\n",
 	      child_name, wname);
@@ -441,43 +451,13 @@ gb_gnome_message_box_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
   /* action_area is a child of vbox so I had to add a kludge to stop it
      being written as a normal child - we need to do it here so that we
      don't output code to create it. */
-  child_name = gtk_widget_get_name (GNOME_DIALOG (widget)->action_area);
+  child_name = (char*) gtk_widget_get_name (GNOME_DIALOG (widget)->action_area);
   child_name = source_create_valid_identifier (child_name);
-
-  data->create_widget = FALSE;
-  gb_widget_write_source (GNOME_DIALOG (widget)->action_area, data);
-
-  /* We need to move this after the above, because GnomeMessageBox doesn't
-     actually create the action_area until a button is added. Note that this
-     means we mustn't set any properties of the action_area above, (see
-     gbhbuttonbox.c), and the GnomeMessageBox must have at least one button. */
   source_add (data, "  %s = GNOME_DIALOG (%s)->action_area;\n",
 	      child_name, wname);
-
-  /* Copied from gb_widget_write_standard_source(), until we decide on a better
-     way to structure this. */
-  source_add_decl (data, "  GtkWidget *%s;\n", child_name);
-
-  /* If there aren't any buttons the pointer may be NULL, so we add a test. */
-  if (data->set_widget_names)
-    source_add (data,
-		"  if (%s != NULL)\n"
-		"    gtk_widget_set_name (%s, \"%s\");\n",
-		child_name, child_name, child_name);
-
-  if (!data->use_component_struct)
-    {
-      source_add (data,
-		  "  gtk_widget_ref (%s);\n"
-		  "  gtk_object_set_data_full (GTK_OBJECT (%s), %s, %s,\n"
-		  "                            (GtkDestroyNotify) gtk_widget_unref);\n",
-		  child_name,
-		  data->component_name,
-		  source_make_string (child_name, FALSE),
-		  child_name);
-    }
-
   g_free (child_name);
+  data->create_widget = FALSE;
+  gb_widget_write_source (GNOME_DIALOG (widget)->action_area, data);
 
   g_free (wname);
   data->write_children = FALSE;
@@ -489,9 +469,9 @@ static GtkWidget *
 gb_gnome_message_box_get_child (GtkWidget * widget,
 				const gchar * child_name)
 {
-  if (!strcmp (child_name, DialogVBox))
+  if (!strcmp (child_name, GladeChildDialogVBox))
     return GNOME_DIALOG (widget)->vbox;
-  else if (!strcmp (child_name, DialogActionArea))
+  else if (!strcmp (child_name, GladeChildDialogActionArea))
     return GNOME_DIALOG (widget)->action_area;
   else
     return NULL;
@@ -507,7 +487,8 @@ GbWidget*
 gb_gnome_message_box_init ()
 {
   /* Initialise the GTK type */
-  gnome_message_box_get_type();
+  volatile GtkType type;
+  type = gnome_message_box_get_type();
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct(&gbwidget);
@@ -523,6 +504,7 @@ gb_gnome_message_box_init ()
   gbwidget.gb_widget_set_properties	= gb_gnome_message_box_set_properties;
   gbwidget.gb_widget_get_child		= gb_gnome_message_box_get_child;
   gbwidget.gb_widget_write_source	= gb_gnome_message_box_write_source;
+  gbwidget.gb_widget_destroy = gb_window_destroy;
 /*
   gbwidget.gb_widget_create_popup_menu	= gb_gnome_message_box_create_popup_menu;
 */
