@@ -18,8 +18,6 @@
 #ifndef GLADE_PROPERTY_H
 #define GLADE_PROPERTY_H
 
-#include <gtk/gtkfeatures.h>
-
 #include "gbwidget.h"
 #include "glade_project.h"
 
@@ -37,11 +35,11 @@ extern "C" {
    of property, to be used when the properties of the different widgets differ.
 */
 
+extern GtkWidget *win_property;
+
 /* These are the standard widget property names */
 extern const gchar*	GbName;
 extern const gchar*	GbClass;
-extern const gchar*	GbX;
-extern const gchar*	GbY;
 extern const gchar*	GbWidth;
 extern const gchar*	GbHeight;
 extern const gchar*	GbVisible;
@@ -96,6 +94,7 @@ extern const gchar*	GbAccelKey;
 extern const gchar*	GbAccelSignal;
 
 
+void	    property_init		(void);
 
 /* Showing/hiding the property editor window. */
 void	    property_show		(GtkWidget	    *widget,
@@ -107,6 +106,9 @@ gint	    property_hide		(GtkWidget	    *widget,
 GtkWidget*  property_get_widget		(void);
 void	    property_set_widget		(GtkWidget	    *widget);
 
+/* Updates the window title, i.e. if the widget's name has been changed. */
+void	    property_update_title	(void);
+
 /* Adding property pages specific to a particular widget class. */
 gint	    property_add_gbwidget_page	(GtkWidget	    *page);
 void	    property_hide_gbwidget_page	(void);
@@ -137,10 +139,14 @@ void	    property_add_string		(const gchar	    *property_name,
 					 const gchar	    *label,
 					 const gchar	    *tooltip);
 gchar*	    property_get_string		(const gchar	    *property_name,
+					 GtkWidget	    *actual_widget,
 					 GtkWidget	    *to_apply,
 					 gboolean	    *apply);
 void	    property_set_string		(const gchar	    *property_name,
 					 const gchar	    *value);
+void	    property_set_translatable_string (const gchar   *property_name,
+					      const gchar   *value,
+					      GtkWidget     *widget);
 
 void	    property_add_text		(const gchar	    *property_name,
 					 const gchar	    *label,
@@ -148,10 +154,14 @@ void	    property_add_text		(const gchar	    *property_name,
 					 gint		     visible_lines);
 /* Note: string returned from property_get_text must be freed with g_free() */
 gchar*	    property_get_text		(const gchar	    *property_name,
+					 GtkWidget	    *actual_widget,
 					 GtkWidget	    *to_apply,
 					 gboolean	    *apply);
 void	    property_set_text		(const gchar	    *property_name,
 					 const gchar	    *value);
+void	    property_set_translatable_text (const gchar     *property_name,
+					    const gchar     *value,
+					    GtkWidget       *widget);
 
 void	    property_add_int		(const gchar	    *property_name,
 					 const gchar	    *label,
@@ -307,6 +317,50 @@ void	    property_add_command	(const gchar	    *property_name,
 					 const gchar	    *command,
 					 GtkSignalFunc	     callback);
 
+/* A stock item is a GTK+ stock item that has a label and an icon with the
+   given size. Any stock items that don't have labels or don't have an icon
+   with the given size are not displayed in the popup list.
+   You can change the icon size dynamically, e.g. based on another property.
+   The popup list will be recreated, and if the stock item currently used does
+   not have the given icon size, the stock item will be set to 'None'.
+   Using GLADE_ICON_SIZE_ANY means any icon size is OK. */
+#define	GLADE_ICON_SIZE_ANY	255
+
+void	    property_add_stock_item	(const gchar	    *property_name,
+					 const gchar	    *label,
+					 const gchar	    *tooltip,
+					 GtkIconSize	     icon_size);
+gchar*	    property_get_stock_item	(const gchar	    *property_name,
+					 GtkWidget	    *to_apply,
+					 gboolean	    *apply);
+void	    property_set_stock_item	(const gchar	    *property_name,
+					 const gchar	    *stock_id);
+void	    property_set_stock_item_icon_size (const gchar  *property_name,
+					       GtkIconSize   icon_size);
+
+
+/* An icon is a GTK+ stock icon with the given icon size, or a user-specified
+   icon. It will show a GtkCombo, with a '...' button beside it for opening
+   a file selection dialog, so the user can specify an icon file.
+   As with stock item, you can change the icon size dynamically, and the popup
+   list of stock icons will be recreated. The file selection can be shown or
+   hidden with property_set_icon_filesel(). */
+void	    property_add_icon		(const gchar	    *property_name,
+					 const gchar	    *label,
+					 const gchar	    *tooltip,
+					 GtkIconSize	     icon_size);
+gchar*	    property_get_icon		(const gchar	    *property_name,
+					 GtkWidget	    *to_apply,
+					 gboolean	    *apply);
+void	    property_set_icon		(const gchar	    *property_name,
+					 const gchar	    *icon);
+void	    property_set_icon_size	(const gchar	    *property_name,
+					 GtkIconSize	     icon_size);
+void	    property_set_icon_filesel	(const gchar	    *property_name,
+					 gboolean	     filesel);
+
+
+
 #define GB_ADJUST_DEFAULT_LABELS 1
 #define GB_ADJUST_H_LABELS	 2
 #define GB_ADJUST_V_LABELS	 3
